# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if getattr(kaitaistruct, 'API_VERSION', (0, 9)) < (0, 9):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Cubebel1(KaitaiStruct):
    """:field dest_callsign: ax25_frame.ax25_header.dest_callsign_raw.callsign_ror.callsign
    :field src_callsign: ax25_frame.ax25_header.src_callsign_raw.callsign_ror.callsign
    :field src_ssid: ax25_frame.ax25_header.src_ssid_raw.ssid
    :field dest_ssid: ax25_frame.ax25_header.dest_ssid_raw.ssid
    :field ctl: ax25_frame.ax25_header.ctl
    :field pid: ax25_frame.payload.pid
    :field hdr_rf_id: ax25_frame.payload.ax25_info.header.rf_id
    :field hdr_opr_time: ax25_frame.payload.ax25_info.header.opr_time
    :field hdr_reboot_cnt: ax25_frame.payload.ax25_info.header.reboot_cnt
    :field hdr_mcusr: ax25_frame.payload.ax25_info.header.mcusr
    :field hdr_pamp_temp: ax25_frame.payload.ax25_info.header.pamp_temp
    :field hdr_pamp_voltage: ax25_frame.payload.ax25_info.header.pamp_voltage
    :field hdr_tx_attenuator: ax25_frame.payload.ax25_info.header.tx_attenuator
    :field hdr_battery_voltage: ax25_frame.payload.ax25_info.header.battery_voltage
    :field hdr_system_voltage: ax25_frame.payload.ax25_info.header.system_voltage
    :field hdr_seq_number: ax25_frame.payload.ax25_info.header.seq_number
    :field hdr_pwr_save_state: ax25_frame.payload.ax25_info.header.pwr_save_state
    :field hdr_modem_on_period: ax25_frame.payload.ax25_info.header.modem_on_period
    :field hdr_obc_can_status: ax25_frame.payload.ax25_info.header.obc_can_status
    :field hdr_eps_can_status: ax25_frame.payload.ax25_info.header.eps_can_status
    :field hdr_info_size: ax25_frame.payload.ax25_info.header.info_size
    :field hdr_data_type: ax25_frame.payload.ax25_info.header.data_type
    :field fec_crc_status: ax25_frame.payload.ax25_info.data.fec_crc_status
    :field rx_msg_state: ax25_frame.payload.ax25_info.data.rx_msg_state
    :field rssi: ax25_frame.payload.ax25_info.data.rssi
    :field rf_msg: ax25_frame.payload.ax25_info.data.rf_msg
    :field current_to_gamma: ax25_frame.payload.ax25_info.data.current_to_gamma
    :field current_to_irsensor: ax25_frame.payload.ax25_info.data.current_to_irsensor
    :field current_to_extflash: ax25_frame.payload.ax25_info.data.current_to_extflash
    :field current_to_solarsens: ax25_frame.payload.ax25_info.data.current_to_solarsens
    :field current_to_magnetcoils: ax25_frame.payload.ax25_info.data.current_to_magnetcoils
    :field current_to_coil_x: ax25_frame.payload.ax25_info.data.current_to_coil_x
    :field current_to_coil_y: ax25_frame.payload.ax25_info.data.current_to_coil_y
    :field current_to_coil_pz: ax25_frame.payload.ax25_info.data.current_to_coil_pz
    :field current_to_coil_nz: ax25_frame.payload.ax25_info.data.current_to_coil_nz
    :field battery1_temp: ax25_frame.payload.ax25_info.data.battery1_temp
    :field battery2_temp: ax25_frame.payload.ax25_info.data.battery2_temp
    :field numb_oc_obc: ax25_frame.payload.ax25_info.data.numb_oc_obc
    :field numb_oc_out_gamma: ax25_frame.payload.ax25_info.data.numb_oc_out_gamma
    :field numb_oc_out_rf1: ax25_frame.payload.ax25_info.data.numb_oc_out_rf1
    :field numb_oc_out_rf2: ax25_frame.payload.ax25_info.data.numb_oc_out_rf2
    :field numb_oc_out_flash: ax25_frame.payload.ax25_info.data.numb_oc_out_flash
    :field numb_oc_out_irsens: ax25_frame.payload.ax25_info.data.numb_oc_out_irsens
    :field numb_oc_coil_x: ax25_frame.payload.ax25_info.data.numb_oc_coil_x
    :field numb_oc_coil_y: ax25_frame.payload.ax25_info.data.numb_oc_coil_y
    :field numb_oc_coil_pz: ax25_frame.payload.ax25_info.data.numb_oc_coil_pz
    :field numb_oc_coil_nz: ax25_frame.payload.ax25_info.data.numb_oc_coil_nz
    :field numb_oc_magnetcoils: ax25_frame.payload.ax25_info.data.numb_oc_magnetcoils
    :field numb_oc_solarsens: ax25_frame.payload.ax25_info.data.numb_oc_solarsens
    :field reset_num: ax25_frame.payload.ax25_info.data.reset_num
    :field reset_reason: ax25_frame.payload.ax25_info.data.reset_reason
    :field pwr_sat: ax25_frame.payload.ax25_info.data.pwr_sat
    :field pwr_rf1: ax25_frame.payload.ax25_info.data.pwr_rf1
    :field pwr_rf2: ax25_frame.payload.ax25_info.data.pwr_rf2
    :field pwr_sunsensor: ax25_frame.payload.ax25_info.data.pwr_sunsensor
    :field pwr_gamma: ax25_frame.payload.ax25_info.data.pwr_gamma
    :field pwr_irsensor: ax25_frame.payload.ax25_info.data.pwr_irsensor
    :field pwr_flash: ax25_frame.payload.ax25_info.data.pwr_flash
    :field pwr_magnet_x: ax25_frame.payload.ax25_info.data.pwr_magnet_x
    :field pwr_magnet_y: ax25_frame.payload.ax25_info.data.pwr_magnet_y
    :field pwr_magnet_z: ax25_frame.payload.ax25_info.data.pwr_magnet_z
    :field sys_time: ax25_frame.payload.ax25_info.data.sys_time
    :field adc_correctness: ax25_frame.payload.ax25_info.data.adc_correctness
    :field t_adc1: ax25_frame.payload.ax25_info.data.t_adc1
    :field t_adc2: ax25_frame.payload.ax25_info.data.t_adc2
    :field stepup_current: ax25_frame.payload.ax25_info.data.stepup_current
    :field stepup_voltage: ax25_frame.payload.ax25_info.data.stepup_voltage
    :field afterbq_current: ax25_frame.payload.ax25_info.data.afterbq_current
    :field battery_voltage: ax25_frame.payload.ax25_info.data.battery_voltage
    :field sys_voltage_50: ax25_frame.payload.ax25_info.data.sys_voltage_50
    :field sys_voltage_33: ax25_frame.payload.ax25_info.data.sys_voltage_33
    :field eps_uc_current: ax25_frame.payload.ax25_info.data.eps_uc_current
    :field obc_uc_current: ax25_frame.payload.ax25_info.data.obc_uc_current
    :field rf1_uc_current: ax25_frame.payload.ax25_info.data.rf1_uc_current
    :field rf2_uc_current: ax25_frame.payload.ax25_info.data.rf2_uc_current
    :field solar_voltage: ax25_frame.payload.ax25_info.data.solar_voltage
    :field side_x_current: ax25_frame.payload.ax25_info.data.side_x_current
    :field side_py_current: ax25_frame.payload.ax25_info.data.side_py_current
    :field side_ny_current: ax25_frame.payload.ax25_info.data.side_ny_current
    :field side_pz_current: ax25_frame.payload.ax25_info.data.side_pz_current
    :field side_nz_current: ax25_frame.payload.ax25_info.data.side_nz_current
    
    .. seealso::
       Source - https://bsusat.com/media/docs/2018/bsusat-1_data_struct.xlsx
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.ax25_frame = Cubebel1.Ax25Frame(self._io, self, self._root)

    class Ax25Frame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ax25_header = Cubebel1.Ax25Header(self._io, self, self._root)
            _on = (self.ax25_header.ctl & 19)
            if _on == 0:
                self.payload = Cubebel1.IFrame(self._io, self, self._root)
            elif _on == 3:
                self.payload = Cubebel1.UiFrame(self._io, self, self._root)
            elif _on == 19:
                self.payload = Cubebel1.UiFrame(self._io, self, self._root)
            elif _on == 16:
                self.payload = Cubebel1.IFrame(self._io, self, self._root)
            elif _on == 18:
                self.payload = Cubebel1.IFrame(self._io, self, self._root)
            elif _on == 2:
                self.payload = Cubebel1.IFrame(self._io, self, self._root)


    class Ax25Header(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.dest_callsign_raw = Cubebel1.CallsignRaw(self._io, self, self._root)
            self.dest_ssid_raw = Cubebel1.SsidMask(self._io, self, self._root)
            self.src_callsign_raw = Cubebel1.CallsignRaw(self._io, self, self._root)
            self.src_ssid_raw = Cubebel1.SsidMask(self._io, self, self._root)
            self.ctl = self._io.read_u1()


    class UiFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            self._raw_ax25_info = self._io.read_bytes_full()
            _io__raw_ax25_info = KaitaiStream(BytesIO(self._raw_ax25_info))
            self.ax25_info = Cubebel1.Frame(_io__raw_ax25_info, self, self._root)


    class Callsign(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.callsign = (self._io.read_bytes(6)).decode(u"ASCII")


    class RfMessage(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.rf_msg = (self._io.read_bytes((self._parent.header.info_size - 2))).decode(u"utf-8")


    class IFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            self.ax25_info = self._io.read_bytes_full()


    class Frame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.header = Cubebel1.Header(self._io, self, self._root)
            if self.header.info_size > 0:
                _on = self.header.data_type
                if _on == 1:
                    self.data = Cubebel1.RfResponse(self._io, self, self._root)
                elif _on == 3:
                    self.data = Cubebel1.RfMessage(self._io, self, self._root)
                elif _on == 254:
                    self.data = Cubebel1.EpsFullTel(self._io, self, self._root)
                elif _on == 255:
                    self.data = Cubebel1.EpsShortTel(self._io, self, self._root)



    class SsidMask(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ssid_mask = self._io.read_u1()

        @property
        def ssid(self):
            if hasattr(self, '_m_ssid'):
                return self._m_ssid

            self._m_ssid = ((self.ssid_mask & 15) >> 1)
            return getattr(self, '_m_ssid', None)


    class EpsShortTel(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.bytes = []
            i = 0
            while not self._io.is_eof():
                self.bytes.append(self._io.read_u1())
                i += 1


        @property
        def eps_uc_current(self):
            """12 bits."""
            if hasattr(self, '_m_eps_uc_current'):
                return self._m_eps_uc_current

            self._m_eps_uc_current = (((self.bytes[14] | (self.bytes[15] << 8)) >> 2) & 4095)
            return getattr(self, '_m_eps_uc_current', None)

        @property
        def side_py_current(self):
            """12 bits."""
            if hasattr(self, '_m_side_py_current'):
                return self._m_side_py_current

            self._m_side_py_current = ((((self.bytes[22] | (self.bytes[23] << 8)) | (self.bytes[24] << 16)) >> 6) & 4095)
            return getattr(self, '_m_side_py_current', None)

        @property
        def t_adc1(self):
            """12 bits."""
            if hasattr(self, '_m_t_adc1'):
                return self._m_t_adc1

            self._m_t_adc1 = (((self.bytes[2] | (self.bytes[3] << 8)) >> 2) & 4095)
            return getattr(self, '_m_t_adc1', None)

        @property
        def afterbq_current(self):
            """12 bits."""
            if hasattr(self, '_m_afterbq_current'):
                return self._m_afterbq_current

            self._m_afterbq_current = (((self.bytes[8] | (self.bytes[9] << 8)) >> 2) & 4095)
            return getattr(self, '_m_afterbq_current', None)

        @property
        def battery_voltage(self):
            """12 bits."""
            if hasattr(self, '_m_battery_voltage'):
                return self._m_battery_voltage

            self._m_battery_voltage = ((((self.bytes[9] | (self.bytes[10] << 8)) | (self.bytes[11] << 16)) >> 6) & 4095)
            return getattr(self, '_m_battery_voltage', None)

        @property
        def sys_voltage_50(self):
            """12 bits."""
            if hasattr(self, '_m_sys_voltage_50'):
                return self._m_sys_voltage_50

            self._m_sys_voltage_50 = (((self.bytes[11] | (self.bytes[12] << 8)) >> 2) & 4095)
            return getattr(self, '_m_sys_voltage_50', None)

        @property
        def stepup_current(self):
            """12 bits."""
            if hasattr(self, '_m_stepup_current'):
                return self._m_stepup_current

            self._m_stepup_current = (((self.bytes[5] | (self.bytes[6] << 8)) >> 2) & 4095)
            return getattr(self, '_m_stepup_current', None)

        @property
        def side_pz_current(self):
            """12 bits."""
            if hasattr(self, '_m_side_pz_current'):
                return self._m_side_pz_current

            self._m_side_pz_current = ((((self.bytes[25] | (self.bytes[26] << 8)) | (self.bytes[27] << 16)) >> 6) & 4095)
            return getattr(self, '_m_side_pz_current', None)

        @property
        def sys_voltage_33(self):
            """12 bits."""
            if hasattr(self, '_m_sys_voltage_33'):
                return self._m_sys_voltage_33

            self._m_sys_voltage_33 = ((((self.bytes[12] | (self.bytes[13] << 8)) | (self.bytes[14] << 16)) >> 6) & 4095)
            return getattr(self, '_m_sys_voltage_33', None)

        @property
        def rf2_uc_current(self):
            """12 bits."""
            if hasattr(self, '_m_rf2_uc_current'):
                return self._m_rf2_uc_current

            self._m_rf2_uc_current = (((self.bytes[18] | (self.bytes[19] << 8)) >> 2) & 4095)
            return getattr(self, '_m_rf2_uc_current', None)

        @property
        def solar_voltage(self):
            """12 bits."""
            if hasattr(self, '_m_solar_voltage'):
                return self._m_solar_voltage

            self._m_solar_voltage = ((((self.bytes[19] | (self.bytes[20] << 8)) | (self.bytes[21] << 16)) >> 6) & 4095)
            return getattr(self, '_m_solar_voltage', None)

        @property
        def side_x_current(self):
            """12 bits."""
            if hasattr(self, '_m_side_x_current'):
                return self._m_side_x_current

            self._m_side_x_current = (((self.bytes[21] | (self.bytes[22] << 8)) >> 2) & 4095)
            return getattr(self, '_m_side_x_current', None)

        @property
        def obc_uc_current(self):
            """10 bits."""
            if hasattr(self, '_m_obc_uc_current'):
                return self._m_obc_uc_current

            self._m_obc_uc_current = ((self.bytes[15] | (self.bytes[16] << 8)) >> 6)
            return getattr(self, '_m_obc_uc_current', None)

        @property
        def side_nz_current(self):
            """12 bits."""
            if hasattr(self, '_m_side_nz_current'):
                return self._m_side_nz_current

            self._m_side_nz_current = (((self.bytes[27] | (self.bytes[28] << 8)) >> 2) & 4095)
            return getattr(self, '_m_side_nz_current', None)

        @property
        def adc_correctness(self):
            """2 bits."""
            if hasattr(self, '_m_adc_correctness'):
                return self._m_adc_correctness

            self._m_adc_correctness = (self.bytes[2] & 3)
            return getattr(self, '_m_adc_correctness', None)

        @property
        def sys_time(self):
            """16 bits."""
            if hasattr(self, '_m_sys_time'):
                return self._m_sys_time

            self._m_sys_time = (self.bytes[0] | (self.bytes[1] << 8))
            return getattr(self, '_m_sys_time', None)

        @property
        def stepup_voltage(self):
            """12 bits."""
            if hasattr(self, '_m_stepup_voltage'):
                return self._m_stepup_voltage

            self._m_stepup_voltage = ((((self.bytes[6] | (self.bytes[7] << 8)) | (self.bytes[8] << 16)) >> 6) & 4095)
            return getattr(self, '_m_stepup_voltage', None)

        @property
        def side_ny_current(self):
            """12 bits."""
            if hasattr(self, '_m_side_ny_current'):
                return self._m_side_ny_current

            self._m_side_ny_current = (((self.bytes[24] | (self.bytes[25] << 8)) >> 2) & 4095)
            return getattr(self, '_m_side_ny_current', None)

        @property
        def t_adc2(self):
            """12 bits."""
            if hasattr(self, '_m_t_adc2'):
                return self._m_t_adc2

            self._m_t_adc2 = ((((self.bytes[3] | (self.bytes[4] << 8)) | (self.bytes[5] << 16)) >> 6) & 4095)
            return getattr(self, '_m_t_adc2', None)

        @property
        def rf1_uc_current(self):
            """10 bits."""
            if hasattr(self, '_m_rf1_uc_current'):
                return self._m_rf1_uc_current

            self._m_rf1_uc_current = ((self.bytes[17] | (self.bytes[18] << 8)) & 1023)
            return getattr(self, '_m_rf1_uc_current', None)


    class RfResponse(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.status_bits = self._io.read_u1()
            self.rssi = self._io.read_u1()

        @property
        def fec_crc_status(self):
            if hasattr(self, '_m_fec_crc_status'):
                return self._m_fec_crc_status

            self._m_fec_crc_status = (self.status_bits & 1)
            return getattr(self, '_m_fec_crc_status', None)

        @property
        def rx_msg_state(self):
            if hasattr(self, '_m_rx_msg_state'):
                return self._m_rx_msg_state

            self._m_rx_msg_state = (self.status_bits >> 1)
            return getattr(self, '_m_rx_msg_state', None)


    class EpsFullTel(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.bytes = []
            i = 0
            while not self._io.is_eof():
                self.bytes.append(self._io.read_u1())
                i += 1


        @property
        def eps_uc_current(self):
            """12 bits."""
            if hasattr(self, '_m_eps_uc_current'):
                return self._m_eps_uc_current

            self._m_eps_uc_current = (((self.bytes[14] | (self.bytes[15] << 8)) >> 2) & 4095)
            return getattr(self, '_m_eps_uc_current', None)

        @property
        def pwr_sat(self):
            """1 bits."""
            if hasattr(self, '_m_pwr_sat'):
                return self._m_pwr_sat

            self._m_pwr_sat = (self.bytes[61] & 1)
            return getattr(self, '_m_pwr_sat', None)

        @property
        def pwr_irsensor(self):
            """1 bits."""
            if hasattr(self, '_m_pwr_irsensor'):
                return self._m_pwr_irsensor

            self._m_pwr_irsensor = ((self.bytes[61] >> 5) & 1)
            return getattr(self, '_m_pwr_irsensor', None)

        @property
        def numb_oc_coil_y(self):
            """8 bits."""
            if hasattr(self, '_m_numb_oc_coil_y'):
                return self._m_numb_oc_coil_y

            self._m_numb_oc_coil_y = self.bytes[53]
            return getattr(self, '_m_numb_oc_coil_y', None)

        @property
        def side_py_current(self):
            """12 bits."""
            if hasattr(self, '_m_side_py_current'):
                return self._m_side_py_current

            self._m_side_py_current = ((((self.bytes[22] | (self.bytes[23] << 8)) | (self.bytes[24] << 16)) >> 6) & 4095)
            return getattr(self, '_m_side_py_current', None)

        @property
        def current_to_solarsens(self):
            """12 bits."""
            if hasattr(self, '_m_current_to_solarsens'):
                return self._m_current_to_solarsens

            self._m_current_to_solarsens = (((self.bytes[33] | (self.bytes[34] << 8)) >> 2) & 4095)
            return getattr(self, '_m_current_to_solarsens', None)

        @property
        def current_to_extflash(self):
            """12 bits."""
            if hasattr(self, '_m_current_to_extflash'):
                return self._m_current_to_extflash

            self._m_current_to_extflash = ((((self.bytes[31] | (self.bytes[32] << 8)) | (self.bytes[33] << 16)) >> 6) & 4095)
            return getattr(self, '_m_current_to_extflash', None)

        @property
        def pwr_magnet_x(self):
            """1 bits."""
            if hasattr(self, '_m_pwr_magnet_x'):
                return self._m_pwr_magnet_x

            self._m_pwr_magnet_x = (self.bytes[61] >> 7)
            return getattr(self, '_m_pwr_magnet_x', None)

        @property
        def numb_oc_solarsens(self):
            """8 bits."""
            if hasattr(self, '_m_numb_oc_solarsens'):
                return self._m_numb_oc_solarsens

            self._m_numb_oc_solarsens = self.bytes[57]
            return getattr(self, '_m_numb_oc_solarsens', None)

        @property
        def t_adc1(self):
            """12 bits."""
            if hasattr(self, '_m_t_adc1'):
                return self._m_t_adc1

            self._m_t_adc1 = (((self.bytes[2] | (self.bytes[3] << 8)) >> 2) & 4095)
            return getattr(self, '_m_t_adc1', None)

        @property
        def afterbq_current(self):
            """12 bits."""
            if hasattr(self, '_m_afterbq_current'):
                return self._m_afterbq_current

            self._m_afterbq_current = (((self.bytes[8] | (self.bytes[9] << 8)) >> 2) & 4095)
            return getattr(self, '_m_afterbq_current', None)

        @property
        def pwr_sunsensor(self):
            """1 bits."""
            if hasattr(self, '_m_pwr_sunsensor'):
                return self._m_pwr_sunsensor

            self._m_pwr_sunsensor = ((self.bytes[61] >> 3) & 1)
            return getattr(self, '_m_pwr_sunsensor', None)

        @property
        def numb_oc_obc(self):
            """8 bits."""
            if hasattr(self, '_m_numb_oc_obc'):
                return self._m_numb_oc_obc

            self._m_numb_oc_obc = self.bytes[46]
            return getattr(self, '_m_numb_oc_obc', None)

        @property
        def current_to_coil_pz(self):
            """12 bits."""
            if hasattr(self, '_m_current_to_coil_pz'):
                return self._m_current_to_coil_pz

            self._m_current_to_coil_pz = (((self.bytes[39] | (self.bytes[40] << 8)) >> 2) & 4095)
            return getattr(self, '_m_current_to_coil_pz', None)

        @property
        def battery_voltage(self):
            """12 bits."""
            if hasattr(self, '_m_battery_voltage'):
                return self._m_battery_voltage

            self._m_battery_voltage = ((((self.bytes[9] | (self.bytes[10] << 8)) | (self.bytes[11] << 16)) >> 6) & 4095)
            return getattr(self, '_m_battery_voltage', None)

        @property
        def pwr_rf1(self):
            """1 bits."""
            if hasattr(self, '_m_pwr_rf1'):
                return self._m_pwr_rf1

            self._m_pwr_rf1 = ((self.bytes[61] >> 1) & 1)
            return getattr(self, '_m_pwr_rf1', None)

        @property
        def sys_voltage_50(self):
            """12 bits."""
            if hasattr(self, '_m_sys_voltage_50'):
                return self._m_sys_voltage_50

            self._m_sys_voltage_50 = (((self.bytes[11] | (self.bytes[12] << 8)) >> 2) & 4095)
            return getattr(self, '_m_sys_voltage_50', None)

        @property
        def numb_oc_magnetcoils(self):
            """8 bits."""
            if hasattr(self, '_m_numb_oc_magnetcoils'):
                return self._m_numb_oc_magnetcoils

            self._m_numb_oc_magnetcoils = self.bytes[56]
            return getattr(self, '_m_numb_oc_magnetcoils', None)

        @property
        def pwr_magnet_z(self):
            """1 bits."""
            if hasattr(self, '_m_pwr_magnet_z'):
                return self._m_pwr_magnet_z

            self._m_pwr_magnet_z = ((self.bytes[62] >> 1) & 1)
            return getattr(self, '_m_pwr_magnet_z', None)

        @property
        def stepup_current(self):
            """12 bits."""
            if hasattr(self, '_m_stepup_current'):
                return self._m_stepup_current

            self._m_stepup_current = (((self.bytes[5] | (self.bytes[6] << 8)) >> 2) & 4095)
            return getattr(self, '_m_stepup_current', None)

        @property
        def side_pz_current(self):
            """12 bits."""
            if hasattr(self, '_m_side_pz_current'):
                return self._m_side_pz_current

            self._m_side_pz_current = ((((self.bytes[25] | (self.bytes[26] << 8)) | (self.bytes[27] << 16)) >> 6) & 4095)
            return getattr(self, '_m_side_pz_current', None)

        @property
        def current_to_coil_y(self):
            """12 bits."""
            if hasattr(self, '_m_current_to_coil_y'):
                return self._m_current_to_coil_y

            self._m_current_to_coil_y = ((((self.bytes[37] | (self.bytes[38] << 8)) | (self.bytes[39] << 16)) >> 6) & 4095)
            return getattr(self, '_m_current_to_coil_y', None)

        @property
        def numb_oc_out_irsens(self):
            """8 bits."""
            if hasattr(self, '_m_numb_oc_out_irsens'):
                return self._m_numb_oc_out_irsens

            self._m_numb_oc_out_irsens = self.bytes[51]
            return getattr(self, '_m_numb_oc_out_irsens', None)

        @property
        def sys_voltage_33(self):
            """12 bits."""
            if hasattr(self, '_m_sys_voltage_33'):
                return self._m_sys_voltage_33

            self._m_sys_voltage_33 = ((((self.bytes[12] | (self.bytes[13] << 8)) | (self.bytes[14] << 16)) >> 6) & 4095)
            return getattr(self, '_m_sys_voltage_33', None)

        @property
        def rf2_uc_current(self):
            """12 bits."""
            if hasattr(self, '_m_rf2_uc_current'):
                return self._m_rf2_uc_current

            self._m_rf2_uc_current = (((self.bytes[18] | (self.bytes[19] << 8)) >> 2) & 4095)
            return getattr(self, '_m_rf2_uc_current', None)

        @property
        def battery2_temp(self):
            """12 bits."""
            if hasattr(self, '_m_battery2_temp'):
                return self._m_battery2_temp

            self._m_battery2_temp = ((((self.bytes[43] | (self.bytes[44] << 8)) | (self.bytes[45] << 16)) >> 6) & 4095)
            return getattr(self, '_m_battery2_temp', None)

        @property
        def current_to_magnetcoils(self):
            """12 bits."""
            if hasattr(self, '_m_current_to_magnetcoils'):
                return self._m_current_to_magnetcoils

            self._m_current_to_magnetcoils = ((((self.bytes[34] | (self.bytes[35] << 8)) | (self.bytes[36] << 16)) >> 6) & 4095)
            return getattr(self, '_m_current_to_magnetcoils', None)

        @property
        def current_to_coil_x(self):
            """12 bits."""
            if hasattr(self, '_m_current_to_coil_x'):
                return self._m_current_to_coil_x

            self._m_current_to_coil_x = (((self.bytes[36] | (self.bytes[37] << 8)) >> 2) & 4095)
            return getattr(self, '_m_current_to_coil_x', None)

        @property
        def numb_oc_out_rf2(self):
            """8 bits."""
            if hasattr(self, '_m_numb_oc_out_rf2'):
                return self._m_numb_oc_out_rf2

            self._m_numb_oc_out_rf2 = self.bytes[49]
            return getattr(self, '_m_numb_oc_out_rf2', None)

        @property
        def solar_voltage(self):
            """12 bits."""
            if hasattr(self, '_m_solar_voltage'):
                return self._m_solar_voltage

            self._m_solar_voltage = ((((self.bytes[19] | (self.bytes[20] << 8)) | (self.bytes[21] << 16)) >> 6) & 4095)
            return getattr(self, '_m_solar_voltage', None)

        @property
        def numb_oc_out_flash(self):
            """8 bits."""
            if hasattr(self, '_m_numb_oc_out_flash'):
                return self._m_numb_oc_out_flash

            self._m_numb_oc_out_flash = self.bytes[50]
            return getattr(self, '_m_numb_oc_out_flash', None)

        @property
        def numb_oc_coil_nz(self):
            """8 bits."""
            if hasattr(self, '_m_numb_oc_coil_nz'):
                return self._m_numb_oc_coil_nz

            self._m_numb_oc_coil_nz = self.bytes[55]
            return getattr(self, '_m_numb_oc_coil_nz', None)

        @property
        def numb_oc_out_gamma(self):
            """8 bits."""
            if hasattr(self, '_m_numb_oc_out_gamma'):
                return self._m_numb_oc_out_gamma

            self._m_numb_oc_out_gamma = self.bytes[47]
            return getattr(self, '_m_numb_oc_out_gamma', None)

        @property
        def side_x_current(self):
            """12 bits."""
            if hasattr(self, '_m_side_x_current'):
                return self._m_side_x_current

            self._m_side_x_current = (((self.bytes[21] | (self.bytes[22] << 8)) >> 2) & 4095)
            return getattr(self, '_m_side_x_current', None)

        @property
        def obc_uc_current(self):
            """10 bits."""
            if hasattr(self, '_m_obc_uc_current'):
                return self._m_obc_uc_current

            self._m_obc_uc_current = ((self.bytes[15] | (self.bytes[16] << 8)) >> 6)
            return getattr(self, '_m_obc_uc_current', None)

        @property
        def current_to_coil_nz(self):
            """12 bits."""
            if hasattr(self, '_m_current_to_coil_nz'):
                return self._m_current_to_coil_nz

            self._m_current_to_coil_nz = ((((self.bytes[40] | (self.bytes[41] << 8)) | (self.bytes[42] << 16)) >> 6) & 4095)
            return getattr(self, '_m_current_to_coil_nz', None)

        @property
        def pwr_flash(self):
            """1 bits."""
            if hasattr(self, '_m_pwr_flash'):
                return self._m_pwr_flash

            self._m_pwr_flash = ((self.bytes[61] >> 6) & 1)
            return getattr(self, '_m_pwr_flash', None)

        @property
        def battery1_temp(self):
            """12 bits."""
            if hasattr(self, '_m_battery1_temp'):
                return self._m_battery1_temp

            self._m_battery1_temp = (((self.bytes[42] | (self.bytes[43] << 8)) >> 2) & 4095)
            return getattr(self, '_m_battery1_temp', None)

        @property
        def current_to_gamma(self):
            """12 bits."""
            if hasattr(self, '_m_current_to_gamma'):
                return self._m_current_to_gamma

            self._m_current_to_gamma = ((((self.bytes[28] | (self.bytes[29] << 8)) | (self.bytes[30] << 16)) >> 6) & 4095)
            return getattr(self, '_m_current_to_gamma', None)

        @property
        def current_to_irsensor(self):
            """12 bits."""
            if hasattr(self, '_m_current_to_irsensor'):
                return self._m_current_to_irsensor

            self._m_current_to_irsensor = (((self.bytes[30] | (self.bytes[31] << 8)) >> 2) & 4095)
            return getattr(self, '_m_current_to_irsensor', None)

        @property
        def side_nz_current(self):
            """12 bits."""
            if hasattr(self, '_m_side_nz_current'):
                return self._m_side_nz_current

            self._m_side_nz_current = (((self.bytes[27] | (self.bytes[28] << 8)) >> 2) & 4095)
            return getattr(self, '_m_side_nz_current', None)

        @property
        def adc_correctness(self):
            """2 bits."""
            if hasattr(self, '_m_adc_correctness'):
                return self._m_adc_correctness

            self._m_adc_correctness = (self.bytes[2] & 3)
            return getattr(self, '_m_adc_correctness', None)

        @property
        def reset_reason(self):
            """8 bits."""
            if hasattr(self, '_m_reset_reason'):
                return self._m_reset_reason

            self._m_reset_reason = self.bytes[60]
            return getattr(self, '_m_reset_reason', None)

        @property
        def sys_time(self):
            """16 bits."""
            if hasattr(self, '_m_sys_time'):
                return self._m_sys_time

            self._m_sys_time = (self.bytes[0] | (self.bytes[1] << 8))
            return getattr(self, '_m_sys_time', None)

        @property
        def pwr_rf2(self):
            """1 bits."""
            if hasattr(self, '_m_pwr_rf2'):
                return self._m_pwr_rf2

            self._m_pwr_rf2 = ((self.bytes[61] >> 2) & 1)
            return getattr(self, '_m_pwr_rf2', None)

        @property
        def numb_oc_coil_pz(self):
            """8 bits."""
            if hasattr(self, '_m_numb_oc_coil_pz'):
                return self._m_numb_oc_coil_pz

            self._m_numb_oc_coil_pz = self.bytes[54]
            return getattr(self, '_m_numb_oc_coil_pz', None)

        @property
        def stepup_voltage(self):
            """12 bits."""
            if hasattr(self, '_m_stepup_voltage'):
                return self._m_stepup_voltage

            self._m_stepup_voltage = ((((self.bytes[6] | (self.bytes[7] << 8)) | (self.bytes[8] << 16)) >> 6) & 4095)
            return getattr(self, '_m_stepup_voltage', None)

        @property
        def side_ny_current(self):
            """12 bits."""
            if hasattr(self, '_m_side_ny_current'):
                return self._m_side_ny_current

            self._m_side_ny_current = (((self.bytes[24] | (self.bytes[25] << 8)) >> 2) & 4095)
            return getattr(self, '_m_side_ny_current', None)

        @property
        def numb_oc_out_rf1(self):
            """8 bits."""
            if hasattr(self, '_m_numb_oc_out_rf1'):
                return self._m_numb_oc_out_rf1

            self._m_numb_oc_out_rf1 = self.bytes[48]
            return getattr(self, '_m_numb_oc_out_rf1', None)

        @property
        def t_adc2(self):
            """12 bits."""
            if hasattr(self, '_m_t_adc2'):
                return self._m_t_adc2

            self._m_t_adc2 = ((((self.bytes[3] | (self.bytes[4] << 8)) | (self.bytes[5] << 16)) >> 6) & 4095)
            return getattr(self, '_m_t_adc2', None)

        @property
        def rf1_uc_current(self):
            """10 bits."""
            if hasattr(self, '_m_rf1_uc_current'):
                return self._m_rf1_uc_current

            self._m_rf1_uc_current = ((self.bytes[17] | (self.bytes[18] << 8)) & 1023)
            return getattr(self, '_m_rf1_uc_current', None)

        @property
        def pwr_gamma(self):
            """1 bits."""
            if hasattr(self, '_m_pwr_gamma'):
                return self._m_pwr_gamma

            self._m_pwr_gamma = ((self.bytes[61] >> 4) & 1)
            return getattr(self, '_m_pwr_gamma', None)

        @property
        def numb_oc_coil_x(self):
            """8 bits."""
            if hasattr(self, '_m_numb_oc_coil_x'):
                return self._m_numb_oc_coil_x

            self._m_numb_oc_coil_x = self.bytes[52]
            return getattr(self, '_m_numb_oc_coil_x', None)

        @property
        def reset_num(self):
            """16 bits."""
            if hasattr(self, '_m_reset_num'):
                return self._m_reset_num

            self._m_reset_num = (self.bytes[58] | (self.bytes[59] << 8))
            return getattr(self, '_m_reset_num', None)

        @property
        def pwr_magnet_y(self):
            """1 bits."""
            if hasattr(self, '_m_pwr_magnet_y'):
                return self._m_pwr_magnet_y

            self._m_pwr_magnet_y = (self.bytes[62] & 1)
            return getattr(self, '_m_pwr_magnet_y', None)


    class Header(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.rf_id = self._io.read_u1()
            self.opr_time = self._io.read_u2le()
            self.reboot_cnt = self._io.read_u1()
            self.mcusr = self._io.read_u1()
            self.pamp_temp = self._io.read_u2le()
            self.pamp_voltage = self._io.read_u1()
            self.tx_attenuator = self._io.read_u1()
            self.battery_voltage = self._io.read_u2le()
            self.system_voltage = self._io.read_u2le()
            self.seq_number = self._io.read_u2le()
            self.pwr_save_state = self._io.read_u1()
            self.modem_on_period = self._io.read_u2le()
            self.obc_can_status = self._io.read_u1()
            self.eps_can_status = self._io.read_u1()
            self.info_size = self._io.read_u1()
            self.data_type = self._io.read_u1()


    class CallsignRaw(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw__raw_callsign_ror = self._io.read_bytes(6)
            self._raw_callsign_ror = KaitaiStream.process_rotate_left(self._raw__raw_callsign_ror, 8 - (1), 1)
            _io__raw_callsign_ror = KaitaiStream(BytesIO(self._raw_callsign_ror))
            self.callsign_ror = Cubebel1.Callsign(_io__raw_callsign_ror, self, self._root)






# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if getattr(kaitaistruct, 'API_VERSION', (0, 9)) < (0, 9):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Knacksat2(KaitaiStruct):
    """:field dest_callsign: id1.id2.ax25_frame.ax25_header.dest_callsign_raw.callsign_ror.callsign
    :field src_callsign: id1.id2.ax25_frame.ax25_header.src_callsign_raw.callsign_ror.callsign
    :field src_ssid: id1.id2.ax25_frame.ax25_header.src_ssid_raw.ssid
    :field dest_ssid: id1.id2.ax25_frame.ax25_header.dest_ssid_raw.ssid
    :field ctl: id1.id2.ax25_frame.ax25_header.ctl
    :field pid: id1.id2.ax25_frame.payload.pid
    :field pcdu_en: id1.id2.ax25_frame.ax25_payload.csp_data.tm_data.pcdu.enable
    :field mppt_r1: id1.id2.ax25_frame.ax25_payload.csp_data.tm_data.solar.mppt.res.res_1
    :field mppt_r2: id1.id2.ax25_frame.ax25_payload.csp_data.tm_data.solar.mppt.res.res_2
    :field mppt_r3: id1.id2.ax25_frame.ax25_payload.csp_data.tm_data.solar.mppt.res.res_3
    :field mppt_power_0: id1.id2.ax25_frame.ax25_payload.csp_data.tm_data.solar.mppt.power_0
    :field mppt_power_1: id1.id2.ax25_frame.ax25_payload.csp_data.tm_data.solar.mppt.power_1
    :field mppt_power_2: id1.id2.ax25_frame.ax25_payload.csp_data.tm_data.solar.mppt.power_2
    :field mppt_power_3: id1.id2.ax25_frame.ax25_payload.csp_data.tm_data.solar.mppt.power_3
    :field mppt_power_4: id1.id2.ax25_frame.ax25_payload.csp_data.tm_data.solar.mppt.power_4
    :field mppt_power_5: id1.id2.ax25_frame.ax25_payload.csp_data.tm_data.solar.mppt.power_5
    :field mppt_power_6: id1.id2.ax25_frame.ax25_payload.csp_data.tm_data.solar.mppt.power_6
    :field mppt_vbus_0: id1.id2.ax25_frame.ax25_payload.csp_data.tm_data.solar.mppt.vbus_0
    :field mppt_vbus_1: id1.id2.ax25_frame.ax25_payload.csp_data.tm_data.solar.mppt.vbus_1
    :field mppt_vbus_2: id1.id2.ax25_frame.ax25_payload.csp_data.tm_data.solar.mppt.vbus_2
    :field mppt_vbus_3: id1.id2.ax25_frame.ax25_payload.csp_data.tm_data.solar.mppt.vbus_3
    :field mppt_vbus_4: id1.id2.ax25_frame.ax25_payload.csp_data.tm_data.solar.mppt.vbus_4
    :field mppt_vbus_5: id1.id2.ax25_frame.ax25_payload.csp_data.tm_data.solar.mppt.vbus_5
    :field mppt_vbus_6: id1.id2.ax25_frame.ax25_payload.csp_data.tm_data.solar.mppt.vbus_6
    :field mppt_isens_0: id1.id2.ax25_frame.ax25_payload.csp_data.tm_data.solar.mppt.isens_0
    :field mppt_isens_1: id1.id2.ax25_frame.ax25_payload.csp_data.tm_data.solar.mppt.isens_1
    :field mppt_isens_2: id1.id2.ax25_frame.ax25_payload.csp_data.tm_data.solar.mppt.isens_2
    :field mppt_isens_3: id1.id2.ax25_frame.ax25_payload.csp_data.tm_data.solar.mppt.isens_3
    :field mppt_isens_4: id1.id2.ax25_frame.ax25_payload.csp_data.tm_data.solar.mppt.isens_4
    :field mppt_isens_5: id1.id2.ax25_frame.ax25_payload.csp_data.tm_data.solar.mppt.isens_5
    :field mppt_isens_6: id1.id2.ax25_frame.ax25_payload.csp_data.tm_data.solar.mppt.isens_6  
    :field batt_r1: id1.id2.ax25_frame.ax25_payload.csp_data.tm_data.battery.res.res_1
    :field batt_r2: id1.id2.ax25_frame.ax25_payload.csp_data.tm_data.battery.res.res_2
    :field batt_r3: id1.id2.ax25_frame.ax25_payload.csp_data.tm_data.battery.res.res_3
    :field batt_heater_cnt: id1.id2.ax25_frame.ax25_payload.csp_data.tm_data.battery.heater_cnt
    :field batt_vout: id1.id2.ax25_frame.ax25_payload.csp_data.tm_data.battery.vout
    :field batt_iout: id1.id2.ax25_frame.ax25_payload.csp_data.tm_data.battery.iout
    :field batt_vin: id1.id2.ax25_frame.ax25_payload.csp_data.tm_data.battery.vin
    :field batt_iin: id1.id2.ax25_frame.ax25_payload.csp_data.tm_data.battery.iin
    :field batt_temp_0: id1.id2.ax25_frame.ax25_payload.csp_data.tm_data.battery.temperature_0
    :field batt_temp_1: id1.id2.ax25_frame.ax25_payload.csp_data.tm_data.battery.temperature_1
    :field batt_temp_2: id1.id2.ax25_frame.ax25_payload.csp_data.tm_data.battery.temperature_2
    :field batt_temp_3: id1.id2.ax25_frame.ax25_payload.csp_data.tm_data.battery.temperature_3
    :field ant_status_1: id1.id2.ax25_frame.ax25_payload.csp_data.tm_data.antenna.isis_ant
    :field ant_status_2: id1.id2.ax25_frame.ax25_payload.csp_data.tm_data.antenna.lora_ant
    :field obc_r1: id1.id2.ax25_frame.ax25_payload.csp_data.tm_data.obc.res.res_1
    :field obc_r2: id1.id2.ax25_frame.ax25_payload.csp_data.tm_data.obc.res.res_2
    :field obc_r3: id1.id2.ax25_frame.ax25_payload.csp_data.tm_data.obc.res.res_3
    :field ident: id1.id2.ax25_frame.ax25_payload.csp_data.tm_data.identify
    :field digi_dest_callsign: id1.id2.ax25frame.ax25_header.dest_callsign_raw.callsign_ror.callsign
    :field digi_src_callsign: id1.id2.ax25frame.ax25_header.src_callsign_raw.callsign_ror.callsign
    :field digi_src_ssid: id1.id2.ax25frame.ax25_header.src_ssid_raw.ssid
    :field digi_dest_ssid: id1.id2.ax25frame.ax25_header.dest_ssid_raw.ssid
    :field rpt_instance___callsign: id1.id2.ax25frame.ax25_header.repeater.rpt_instance.___.rpt_callsign_raw.callsign_ror.callsign
    :field rpt_instance___ssid: id1.id2.ax25frame.ax25_header.repeater.rpt_instance.___.rpt_ssid_raw.ssid
    :field rpt_instance___hbit: id1.id2.ax25frame.ax25_header.repeater.rpt_instance.___.rpt_ssid_raw.hbit
    :field digi_ctl: id1.id2.ax25frame.ax25_header.ctl
    :field digi_pid: id1.id2.ax25frame.ax25_header.pid
    :field digi_message: id1.id2.ax25frame.digi_message
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.id1 = Knacksat2.Type1(self._io, self, self._root)

    class Type1(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            _on = self.message_type1
            if _on == 2426822786:
                self.id2 = Knacksat2.Telemetry(self._io, self, self._root)
            else:
                self.id2 = Knacksat2.Digi(self._io, self, self._root)

        @property
        def message_type1(self):
            """KNACKSAT-2 Telemetry validation
            """
            if hasattr(self, '_m_message_type1'):
                return self._m_message_type1

            _pos = self._io.pos()
            self._io.seek(0)
            self._m_message_type1 = self._io.read_u4be()
            self._io.seek(_pos)
            if not self.message_type1 == 2426822786:
                raise kaitaistruct.ValidationNotEqualError(2426822786, self.message_type1, self._io, u"/types/type1/instances/message_type1")
            return getattr(self, '_m_message_type1', None)


    class Telemetry(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ax25_frame = Knacksat2.Telemetry.Ax25Frame(self._io, self, self._root)

        class Ax25Frame(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.ax25_header = Knacksat2.Telemetry.Ax25Header(self._io, self, self._root)
                self.ax25_payload = Knacksat2.Telemetry.Ax25Payload(self._io, self, self._root)


        class Ax25Payload(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.csp_data = Knacksat2.Telemetry.CspDataT(self._io, self, self._root)


        class Ax25Header(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.dest_callsign_raw = Knacksat2.Telemetry.CallsignRaw(self._io, self, self._root)
                self.dest_ssid_raw = Knacksat2.Telemetry.SsidMask(self._io, self, self._root)
                self.src_callsign_raw = Knacksat2.Telemetry.CallsignRaw(self._io, self, self._root)
                self.src_ssid_raw = Knacksat2.Telemetry.SsidMask(self._io, self, self._root)
                self.ctl = self._io.read_u1()
                self.pid = self._io.read_u1()


        class Callsign(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.callsign = (self._io.read_bytes(6)).decode(u"ASCII")


        class ResT(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.csp_flags = []
                for i in range(5):
                    self.csp_flags.append(self._io.read_u1())


            @property
            def source(self):
                if hasattr(self, '_m_source'):
                    return self._m_source

                self._m_source = ((self.csp_flags[3] >> 1) & 31)
                return getattr(self, '_m_source', None)

            @property
            def res_1(self):
                if hasattr(self, '_m_res_1'):
                    return self._m_res_1

                self._m_res_1 = self.csp_flags[0]
                return getattr(self, '_m_res_1', None)

            @property
            def rdp(self):
                if hasattr(self, '_m_rdp'):
                    return self._m_rdp

                self._m_rdp = ((self.csp_flags[0] >> 1) & 1)
                return getattr(self, '_m_rdp', None)

            @property
            def src_port(self):
                if hasattr(self, '_m_src_port'):
                    return self._m_src_port

                self._m_src_port = (self.csp_flags[1] & 63)
                return getattr(self, '_m_src_port', None)

            @property
            def destination(self):
                if hasattr(self, '_m_destination'):
                    return self._m_destination

                self._m_destination = (((self.csp_flags[2] | (self.csp_flags[3] << 8)) >> 4) & 31)
                return getattr(self, '_m_destination', None)

            @property
            def res_3(self):
                if hasattr(self, '_m_res_3'):
                    return self._m_res_3

                self._m_res_3 = ((self.csp_flags[3] << 8) | self.csp_flags[4])
                return getattr(self, '_m_res_3', None)

            @property
            def dst_port(self):
                if hasattr(self, '_m_dst_port'):
                    return self._m_dst_port

                self._m_dst_port = (((self.csp_flags[1] | (self.csp_flags[2] << 8)) >> 6) & 63)
                return getattr(self, '_m_dst_port', None)

            @property
            def priority(self):
                if hasattr(self, '_m_priority'):
                    return self._m_priority

                self._m_priority = (self.csp_flags[3] >> 6)
                return getattr(self, '_m_priority', None)

            @property
            def reserved(self):
                if hasattr(self, '_m_reserved'):
                    return self._m_reserved

                self._m_reserved = (self.csp_flags[0] >> 4)
                return getattr(self, '_m_reserved', None)

            @property
            def res_2(self):
                if hasattr(self, '_m_res_2'):
                    return self._m_res_2

                self._m_res_2 = ((self.csp_flags[1] << 8) | self.csp_flags[2])
                return getattr(self, '_m_res_2', None)

            @property
            def xtea(self):
                if hasattr(self, '_m_xtea'):
                    return self._m_xtea

                self._m_xtea = ((self.csp_flags[0] >> 2) & 1)
                return getattr(self, '_m_xtea', None)

            @property
            def hmac(self):
                if hasattr(self, '_m_hmac'):
                    return self._m_hmac

                self._m_hmac = ((self.csp_flags[0] >> 3) & 1)
                return getattr(self, '_m_hmac', None)

            @property
            def crc(self):
                if hasattr(self, '_m_crc'):
                    return self._m_crc

                self._m_crc = (self.csp_flags[0] & 1)
                return getattr(self, '_m_crc', None)


        class PcduT(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.res = Knacksat2.Telemetry.ResT(self._io, self, self._root)
                self.enable = self._io.read_u1()


        class CspDataT(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.tm_data = Knacksat2.Telemetry.TmDataT(self._io, self, self._root)


        class CommT(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.res = Knacksat2.Telemetry.ResT(self._io, self, self._root)


        class SsidMask(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.ssid_mask = self._io.read_u1()

            @property
            def ssid(self):
                if hasattr(self, '_m_ssid'):
                    return self._m_ssid

                self._m_ssid = ((self.ssid_mask & 31) >> 1)
                return getattr(self, '_m_ssid', None)


        class ObcT(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.res = Knacksat2.Telemetry.ResT(self._io, self, self._root)


        class TmDataT(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.pcdu = Knacksat2.Telemetry.PcduT(self._io, self, self._root)
                self.comm = Knacksat2.Telemetry.CommT(self._io, self, self._root)
                self.solar = Knacksat2.Telemetry.SolarT(self._io, self, self._root)
                self.battery = Knacksat2.Telemetry.BatteryT(self._io, self, self._root)
                self.antenna = Knacksat2.Telemetry.AntennaT(self._io, self, self._root)
                self.obc = Knacksat2.Telemetry.ObcT(self._io, self, self._root)
                self.identify = (self._io.read_bytes(3)).decode(u"ascii")


        class SolarT(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.mppt = Knacksat2.Telemetry.MpptT(self._io, self, self._root)


        class AntennaT(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.antenna_status = []
                for i in range(2):
                    self.antenna_status.append(self._io.read_u1())


            @property
            def isis_ant(self):
                if hasattr(self, '_m_isis_ant'):
                    return self._m_isis_ant

                self._m_isis_ant = self.antenna_status[0]
                return getattr(self, '_m_isis_ant', None)

            @property
            def lora_ant(self):
                if hasattr(self, '_m_lora_ant'):
                    return self._m_lora_ant

                self._m_lora_ant = self.antenna_status[1]
                return getattr(self, '_m_lora_ant', None)


        class CallsignRaw(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self._raw__raw_callsign_ror = self._io.read_bytes(6)
                self._raw_callsign_ror = KaitaiStream.process_rotate_left(self._raw__raw_callsign_ror, 8 - (1), 1)
                _io__raw_callsign_ror = KaitaiStream(BytesIO(self._raw_callsign_ror))
                self.callsign_ror = Knacksat2.Telemetry.Callsign(_io__raw_callsign_ror, self, self._root)


        class MpptT(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.res = Knacksat2.Telemetry.ResT(self._io, self, self._root)
                self.isens = []
                for i in range(7):
                    self.isens.append(self._io.read_f4le())

                self.power = []
                for i in range(7):
                    self.power.append(self._io.read_f4le())

                self.vbus = []
                for i in range(7):
                    self.vbus.append(self._io.read_f4le())


            @property
            def isens_2(self):
                if hasattr(self, '_m_isens_2'):
                    return self._m_isens_2

                self._m_isens_2 = self.isens[2]
                return getattr(self, '_m_isens_2', None)

            @property
            def power_3(self):
                if hasattr(self, '_m_power_3'):
                    return self._m_power_3

                self._m_power_3 = self.power[3]
                return getattr(self, '_m_power_3', None)

            @property
            def power_4(self):
                if hasattr(self, '_m_power_4'):
                    return self._m_power_4

                self._m_power_4 = self.power[4]
                return getattr(self, '_m_power_4', None)

            @property
            def isens_0(self):
                if hasattr(self, '_m_isens_0'):
                    return self._m_isens_0

                self._m_isens_0 = self.isens[0]
                return getattr(self, '_m_isens_0', None)

            @property
            def power_2(self):
                if hasattr(self, '_m_power_2'):
                    return self._m_power_2

                self._m_power_2 = self.power[2]
                return getattr(self, '_m_power_2', None)

            @property
            def isens_1(self):
                if hasattr(self, '_m_isens_1'):
                    return self._m_isens_1

                self._m_isens_1 = self.isens[1]
                return getattr(self, '_m_isens_1', None)

            @property
            def isens_5(self):
                if hasattr(self, '_m_isens_5'):
                    return self._m_isens_5

                self._m_isens_5 = self.isens[5]
                return getattr(self, '_m_isens_5', None)

            @property
            def vbus_3(self):
                if hasattr(self, '_m_vbus_3'):
                    return self._m_vbus_3

                self._m_vbus_3 = self.vbus[3]
                return getattr(self, '_m_vbus_3', None)

            @property
            def vbus_1(self):
                if hasattr(self, '_m_vbus_1'):
                    return self._m_vbus_1

                self._m_vbus_1 = self.vbus[1]
                return getattr(self, '_m_vbus_1', None)

            @property
            def power_5(self):
                if hasattr(self, '_m_power_5'):
                    return self._m_power_5

                self._m_power_5 = self.power[5]
                return getattr(self, '_m_power_5', None)

            @property
            def isens_4(self):
                if hasattr(self, '_m_isens_4'):
                    return self._m_isens_4

                self._m_isens_4 = self.isens[4]
                return getattr(self, '_m_isens_4', None)

            @property
            def isens_6(self):
                if hasattr(self, '_m_isens_6'):
                    return self._m_isens_6

                self._m_isens_6 = self.isens[6]
                return getattr(self, '_m_isens_6', None)

            @property
            def vbus_5(self):
                if hasattr(self, '_m_vbus_5'):
                    return self._m_vbus_5

                self._m_vbus_5 = self.vbus[5]
                return getattr(self, '_m_vbus_5', None)

            @property
            def vbus_6(self):
                if hasattr(self, '_m_vbus_6'):
                    return self._m_vbus_6

                self._m_vbus_6 = self.vbus[6]
                return getattr(self, '_m_vbus_6', None)

            @property
            def isens_3(self):
                if hasattr(self, '_m_isens_3'):
                    return self._m_isens_3

                self._m_isens_3 = self.isens[3]
                return getattr(self, '_m_isens_3', None)

            @property
            def vbus_2(self):
                if hasattr(self, '_m_vbus_2'):
                    return self._m_vbus_2

                self._m_vbus_2 = self.vbus[2]
                return getattr(self, '_m_vbus_2', None)

            @property
            def power_6(self):
                if hasattr(self, '_m_power_6'):
                    return self._m_power_6

                self._m_power_6 = self.power[6]
                return getattr(self, '_m_power_6', None)

            @property
            def vbus_0(self):
                if hasattr(self, '_m_vbus_0'):
                    return self._m_vbus_0

                self._m_vbus_0 = self.vbus[0]
                return getattr(self, '_m_vbus_0', None)

            @property
            def power_0(self):
                if hasattr(self, '_m_power_0'):
                    return self._m_power_0

                self._m_power_0 = self.power[0]
                return getattr(self, '_m_power_0', None)

            @property
            def power_1(self):
                if hasattr(self, '_m_power_1'):
                    return self._m_power_1

                self._m_power_1 = self.power[1]
                return getattr(self, '_m_power_1', None)

            @property
            def vbus_4(self):
                if hasattr(self, '_m_vbus_4'):
                    return self._m_vbus_4

                self._m_vbus_4 = self.vbus[4]
                return getattr(self, '_m_vbus_4', None)


        class BatteryT(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.res = Knacksat2.Telemetry.ResT(self._io, self, self._root)
                self.heater_cnt = self._io.read_u2le()
                self.temperature = []
                for i in range(4):
                    self.temperature.append(self._io.read_f4le())

                self.vout = self._io.read_f4le()
                self.iout = self._io.read_f4le()
                self.vin = self._io.read_f4le()
                self.iin = self._io.read_f4le()

            @property
            def temperature_0(self):
                if hasattr(self, '_m_temperature_0'):
                    return self._m_temperature_0

                self._m_temperature_0 = self.temperature[0]
                return getattr(self, '_m_temperature_0', None)

            @property
            def temperature_1(self):
                if hasattr(self, '_m_temperature_1'):
                    return self._m_temperature_1

                self._m_temperature_1 = self.temperature[1]
                return getattr(self, '_m_temperature_1', None)

            @property
            def temperature_2(self):
                if hasattr(self, '_m_temperature_2'):
                    return self._m_temperature_2

                self._m_temperature_2 = self.temperature[2]
                return getattr(self, '_m_temperature_2', None)

            @property
            def temperature_3(self):
                if hasattr(self, '_m_temperature_3'):
                    return self._m_temperature_3

                self._m_temperature_3 = self.temperature[3]
                return getattr(self, '_m_temperature_3', None)



    class Digi(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ax25frame = Knacksat2.Digi.Ax25frame(self._io, self, self._root)

        class Ax25Header(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.dest_callsign_raw = Knacksat2.Digi.CallsignRaw(self._io, self, self._root)
                self.dest_ssid_raw = Knacksat2.Digi.SsidMask(self._io, self, self._root)
                self.src_callsign_raw = Knacksat2.Digi.CallsignRaw(self._io, self, self._root)
                self.src_ssid_raw = Knacksat2.Digi.SsidMask(self._io, self, self._root)
                if (self.src_ssid_raw.ssid_mask & 1) == 0:
                    self.repeater = Knacksat2.Digi.Repeater(self._io, self, self._root)

                self.ctl = self._io.read_u1()
                self.pid = self._io.read_u1()


        class Callsign(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.callsign = (self._io.read_bytes(6)).decode(u"ASCII")


        class Ax25frame(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.ax25_header = Knacksat2.Digi.Ax25Header(self._io, self, self._root)
                self.digi_message = (self._io.read_bytes_full()).decode(u"utf-8")


        class SsidMask(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.ssid_mask = self._io.read_u1()

            @property
            def ssid(self):
                if hasattr(self, '_m_ssid'):
                    return self._m_ssid

                self._m_ssid = ((self.ssid_mask & 31) >> 1)
                return getattr(self, '_m_ssid', None)

            @property
            def hbit(self):
                if hasattr(self, '_m_hbit'):
                    return self._m_hbit

                self._m_hbit = ((self.ssid_mask & 128) >> 7)
                return getattr(self, '_m_hbit', None)


        class Repeaters(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.rpt_callsign_raw = Knacksat2.Digi.CallsignRaw(self._io, self, self._root)
                self.rpt_ssid_raw = Knacksat2.Digi.SsidMask(self._io, self, self._root)


        class Repeater(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.rpt_instance = []
                i = 0
                while True:
                    _ = Knacksat2.Digi.Repeaters(self._io, self, self._root)
                    self.rpt_instance.append(_)
                    if (_.rpt_ssid_raw.ssid_mask & 1) == 1:
                        break
                    i += 1


        class CallsignRaw(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self._raw__raw_callsign_ror = self._io.read_bytes(6)
                self._raw_callsign_ror = KaitaiStream.process_rotate_left(self._raw__raw_callsign_ror, 8 - (1), 1)
                _io__raw_callsign_ror = KaitaiStream(BytesIO(self._raw_callsign_ror))
                self.callsign_ror = Knacksat2.Digi.Callsign(_io__raw_callsign_ror, self, self._root)







# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if getattr(kaitaistruct, 'API_VERSION', (0, 9)) < (0, 9):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Bugsat1(KaitaiStruct):
    """:field dest_callsign: ax25_frame.ax25_header.dest_callsign_raw.callsign_ror.callsign
    :field src_callsign: ax25_frame.ax25_header.src_callsign_raw.callsign_ror.callsign
    :field src_ssid: ax25_frame.ax25_header.src_ssid_raw.ssid
    :field dest_ssid: ax25_frame.ax25_header.dest_ssid_raw.ssid
    :field rpt_callsign: ax25_frame.ax25_header.repeater.rpt_instance[0].rpt_callsign_raw.callsign_ror.callsign
    :field ctl: ax25_frame.ax25_header.ctl
    :field pid: ax25_frame.payload.pid
    :field uptime_s: ax25_frame.payload.ax25_info.beacon_type.uptime_s
    :field rtc_s: ax25_frame.payload.ax25_info.beacon_type.rtc_s
    :field reset_count: ax25_frame.payload.ax25_info.beacon_type.reset_count_be
    :field current_mode: ax25_frame.payload.ax25_info.beacon_type.current_mode
    :field last_boot_reason: ax25_frame.payload.ax25_info.beacon_type.last_boot_reason
    :field free: ax25_frame.payload.ax25_info.beacon_type.free
    :field last_seen_sequence_number: ax25_frame.payload.ax25_info.beacon_type.last_seen_sequence_number
    :field antenna_deploy_status: ax25_frame.payload.ax25_info.beacon_type.antenna_deploy_status
    :field low_voltage_counter: ax25_frame.payload.ax25_info.beacon_type.low_voltage_counter
    :field nice_battery_mv: ax25_frame.payload.ax25_info.beacon_type.nice_battery_mv
    :field raw_battery_mv: ax25_frame.payload.ax25_info.beacon_type.raw_battery_mv
    :field battery_amps: ax25_frame.payload.ax25_info.beacon_type.battery_amps
    :field pcm_3v3_v: ax25_frame.payload.ax25_info.beacon_type.pcm_3v3_v
    :field pcm_3v3_a: ax25_frame.payload.ax25_info.beacon_type.pcm_3v3_a
    :field pcm_5v_v: ax25_frame.payload.ax25_info.beacon_type.pcm_5v_v
    :field pcm_5v_a: ax25_frame.payload.ax25_info.beacon_type.pcm_5v_a
    :field cpu_c: ax25_frame.payload.ax25_info.beacon_type.cpu_c
    :field mirror_cell_c: ax25_frame.payload.ax25_info.beacon_type.mirror_cell_c
    :field mode: ax25_frame.payload.ax25_info.beacon_type.mode
    :field sun_vector_x: ax25_frame.payload.ax25_info.beacon_type.sun_vector_x
    :field sun_vector_y: ax25_frame.payload.ax25_info.beacon_type.sun_vector_y
    :field sun_vector_z: ax25_frame.payload.ax25_info.beacon_type.sun_vector_z
    :field magnetometer_x_mg: ax25_frame.payload.ax25_info.beacon_type.magnetometer_x_mg
    :field magnetometer_y_mg: ax25_frame.payload.ax25_info.beacon_type.magnetometer_y_mg
    :field magnetometer_z_mg: ax25_frame.payload.ax25_info.beacon_type.magnetometer_z_mg
    :field gyro_x_dps: ax25_frame.payload.ax25_info.beacon_type.gyro_x_dps
    :field gyro_y_dps: ax25_frame.payload.ax25_info.beacon_type.gyro_y_dps
    :field gyro_z_dps: ax25_frame.payload.ax25_info.beacon_type.gyro_z_dps
    :field temperature_imu_c: ax25_frame.payload.ax25_info.beacon_type.temperature_imu_c
    :field fine_gyro_x_dps: ax25_frame.payload.ax25_info.beacon_type.fine_gyro_x_dps
    :field fine_gyro_y_dps: ax25_frame.payload.ax25_info.beacon_type.fine_gyro_y_dps
    :field fine_gyro_z_dps: ax25_frame.payload.ax25_info.beacon_type.fine_gyro_z_dps
    :field wheel_1_radsec: ax25_frame.payload.ax25_info.beacon_type.wheel_1_radsec
    :field wheel_2_radsec: ax25_frame.payload.ax25_info.beacon_type.wheel_2_radsec
    :field wheel_3_radsec: ax25_frame.payload.ax25_info.beacon_type.wheel_3_radsec
    :field wheel_4_radsec: ax25_frame.payload.ax25_info.beacon_type.wheel_4_radsec
    :field experiments_run: ax25_frame.payload.ax25_info.beacon_type.experiments_run
    :field experiments_failed: ax25_frame.payload.ax25_info.beacon_type.experiments_failed
    :field last_experiment_run: ax25_frame.payload.ax25_info.beacon_type.last_experiment_run
    :field current_state: ax25_frame.payload.ax25_info.beacon_type.current_state
    :field message: ax25_frame.payload.ax25_info.beacon_type.message
    
    Attention: `rpt_callsign` cannot be accessed because `rpt_instance` is an
    array of unknown size at the beginning of the parsing process! Left an
    example in here.
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.ax25_frame = Bugsat1.Ax25Frame(self._io, self, self._root)

    class Ax25Frame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ax25_header = Bugsat1.Ax25Header(self._io, self, self._root)
            _on = (self.ax25_header.ctl & 19)
            if _on == 0:
                self.payload = Bugsat1.IFrame(self._io, self, self._root)
            elif _on == 3:
                self.payload = Bugsat1.UiFrame(self._io, self, self._root)
            elif _on == 19:
                self.payload = Bugsat1.UiFrame(self._io, self, self._root)
            elif _on == 16:
                self.payload = Bugsat1.IFrame(self._io, self, self._root)
            elif _on == 18:
                self.payload = Bugsat1.IFrame(self._io, self, self._root)
            elif _on == 2:
                self.payload = Bugsat1.IFrame(self._io, self, self._root)


    class Ax25Header(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.dest_callsign_raw = Bugsat1.CallsignRaw(self._io, self, self._root)
            self.dest_ssid_raw = Bugsat1.SsidMask(self._io, self, self._root)
            self.src_callsign_raw = Bugsat1.CallsignRaw(self._io, self, self._root)
            self.src_ssid_raw = Bugsat1.SsidMask(self._io, self, self._root)
            if (self.src_ssid_raw.ssid_mask & 1) == 0:
                self.repeater = Bugsat1.Repeater(self._io, self, self._root)

            self.ctl = self._io.read_u1()


    class UiFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            self._raw_ax25_info = self._io.read_bytes_full()
            _io__raw_ax25_info = KaitaiStream(BytesIO(self._raw_ax25_info))
            self.ax25_info = Bugsat1.Ax25InfoData(_io__raw_ax25_info, self, self._root)


    class Callsign(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.callsign = (self._io.read_bytes(6)).decode(u"ASCII")


    class IFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            self._raw_ax25_info = self._io.read_bytes_full()
            _io__raw_ax25_info = KaitaiStream(BytesIO(self._raw_ax25_info))
            self.ax25_info = Bugsat1.Ax25InfoData(_io__raw_ax25_info, self, self._root)


    class SsidMask(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ssid_mask = self._io.read_u1()

        @property
        def ssid(self):
            if hasattr(self, '_m_ssid'):
                return self._m_ssid

            self._m_ssid = ((self.ssid_mask & 15) >> 1)
            return getattr(self, '_m_ssid', None)


    class Repeaters(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.rpt_callsign_raw = Bugsat1.CallsignRaw(self._io, self, self._root)
            self.rpt_ssid_raw = Bugsat1.SsidMask(self._io, self, self._root)


    class Repeater(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.rpt_instance = []
            i = 0
            while True:
                _ = Bugsat1.Repeaters(self._io, self, self._root)
                self.rpt_instance.append(_)
                if (_.rpt_ssid_raw.ssid_mask & 1) == 1:
                    break
                i += 1


    class Message(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.message = (self._io.read_bytes_full()).decode(u"utf-8")


    class CallsignRaw(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw__raw_callsign_ror = self._io.read_bytes(6)
            self._raw_callsign_ror = KaitaiStream.process_rotate_left(self._raw__raw_callsign_ror, 8 - (1), 1)
            _io__raw_callsign_ror = KaitaiStream(BytesIO(self._raw_callsign_ror))
            self.callsign_ror = Bugsat1.Callsign(_io__raw_callsign_ror, self, self._root)


    class Ax25InfoData(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.beacon_id0 = self._io.read_u1()
            if self.beacon_id0 == 255:
                self.beacon_id1 = self._io.read_bytes(1)
                if not self.beacon_id1 == b"\xFF":
                    raise kaitaistruct.ValidationNotEqualError(b"\xFF", self.beacon_id1, self._io, u"/types/ax25_info_data/seq/1")

            if self.beacon_id0 == 255:
                self.beacon_id2 = self._io.read_bytes(1)
                if not self.beacon_id2 == b"\xF0":
                    raise kaitaistruct.ValidationNotEqualError(b"\xF0", self.beacon_id2, self._io, u"/types/ax25_info_data/seq/2")

            _on = self.beacon_id0
            if _on == 58:
                self.beacon_type = Bugsat1.Message(self._io, self, self._root)
            else:
                self.beacon_type = Bugsat1.Telemetry(self._io, self, self._root)


    class Telemetry(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.platform_id = self._io.read_bytes(2)
            if not self.platform_id == b"\x00\x01":
                raise kaitaistruct.ValidationNotEqualError(b"\x00\x01", self.platform_id, self._io, u"/types/telemetry/seq/0")
            self.uptime_s = self._io.read_u4be()
            self.rtc_s = self._io.read_u4be()
            self.rstcnt_b = []
            for i in range(3):
                self.rstcnt_b.append(self._io.read_u1())

            self.current_mode = self._io.read_u1()
            self.last_boot_reason = self._io.read_u4be()
            self.mem_tlm_id = self._io.read_bytes(2)
            if not self.mem_tlm_id == b"\x01\x01":
                raise kaitaistruct.ValidationNotEqualError(b"\x01\x01", self.mem_tlm_id, self._io, u"/types/telemetry/seq/6")
            self.free = self._io.read_u4be()
            self.cdh_id = self._io.read_bytes(2)
            if not self.cdh_id == b"\x02\x01":
                raise kaitaistruct.ValidationNotEqualError(b"\x02\x01", self.cdh_id, self._io, u"/types/telemetry/seq/8")
            self.last_seen_sequence_number = self._io.read_u4be()
            self.antenna_deploy_status = self._io.read_u1()
            self.pwr_tlm_id = self._io.read_bytes(2)
            if not self.pwr_tlm_id == b"\x03\x01":
                raise kaitaistruct.ValidationNotEqualError(b"\x03\x01", self.pwr_tlm_id, self._io, u"/types/telemetry/seq/11")
            self.low_voltage_counter = self._io.read_u2be()
            self.nice_battery_mv = self._io.read_u2be()
            self.raw_battery_mv = self._io.read_u2be()
            self.battery_amps = self._io.read_u2be()
            self.pcm_3v3_v = self._io.read_u2be()
            self.pcm_3v3_a = self._io.read_u2be()
            self.pcm_5v_v = self._io.read_u2be()
            self.pcm_5v_a = self._io.read_u2be()
            self.thermal_tlm_id = self._io.read_bytes(2)
            if not self.thermal_tlm_id == b"\x04\x01":
                raise kaitaistruct.ValidationNotEqualError(b"\x04\x01", self.thermal_tlm_id, self._io, u"/types/telemetry/seq/20")
            self.cpu_c = self._io.read_s2be()
            self.mirror_cell_c = self._io.read_s2be()
            self.aocs_tlm_id = self._io.read_bytes(2)
            if not self.aocs_tlm_id == b"\x05\x01":
                raise kaitaistruct.ValidationNotEqualError(b"\x05\x01", self.aocs_tlm_id, self._io, u"/types/telemetry/seq/23")
            self.mode = self._io.read_u4be()
            self.sun_vector_x = self._io.read_s2be()
            self.sun_vector_y = self._io.read_s2be()
            self.sun_vector_z = self._io.read_s2be()
            self.magnetometer_x_mg = self._io.read_s2be()
            self.magnetometer_y_mg = self._io.read_s2be()
            self.magnetometer_z_mg = self._io.read_s2be()
            self.gyro_x_dps = self._io.read_s2be()
            self.gyro_y_dps = self._io.read_s2be()
            self.gyro_z_dps = self._io.read_s2be()
            self.temperature_imu_c = self._io.read_s2be()
            self.fine_gyro_x_dps = self._io.read_s4be()
            self.fine_gyro_y_dps = self._io.read_s4be()
            self.fine_gyro_z_dps = self._io.read_s4be()
            self.wheel_1_radsec = self._io.read_s2be()
            self.wheel_2_radsec = self._io.read_s2be()
            self.wheel_3_radsec = self._io.read_s2be()
            self.wheel_4_radsec = self._io.read_s2be()
            self.payload_tlm_id = self._io.read_bytes(2)
            if not self.payload_tlm_id == b"\x06\x01":
                raise kaitaistruct.ValidationNotEqualError(b"\x06\x01", self.payload_tlm_id, self._io, u"/types/telemetry/seq/42")
            self.experiments_run = self._io.read_u2be()
            self.experiments_failed = self._io.read_u2be()
            self.last_experiment_run = self._io.read_s2be()
            self.current_state = self._io.read_u1()

        @property
        def reset_count_be(self):
            if hasattr(self, '_m_reset_count_be'):
                return self._m_reset_count_be

            self._m_reset_count_be = (((self.rstcnt_b[0] << 16) | (self.rstcnt_b[1] << 8)) | self.rstcnt_b[2])
            return getattr(self, '_m_reset_count_be', None)






# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if getattr(kaitaistruct, 'API_VERSION', (0, 9)) < (0, 9):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Fo29(KaitaiStruct):
    """:field header_hihi: header_hihi
    :field main_relay: main_relay
    :field dcm: dcm
    :field sram: sram
    :field packet: packet
    :field jta: jta
    :field jtd: jtd
    :field geomagnetism_sensor: geomagnetism_sensor
    :field sun_sensor: sun_sensor
    :field uvc: uvc
    :field uvc_level: uvc_level
    :field pcu_mode: pcu_mode
    :field pcu_level: pcu_level
    :field battery_mode: battery_mode
    :field battery_logic: battery_logic
    :field data_collect_mode: data_collect_mode
    :field repro_mode: repro_mode
    :field packet_hk_date_mode: packet_hk_date_mode
    :field packet_date_collect_mode: packet_date_collect_mode
    :field digitalker_mode: digitalker_mode
    :field double_cmd_ena_dis: double_cmd_ena_dis
    :field uvc_active_passive: uvc_active_passive
    :field cpu_run_reset: cpu_run_reset
    :field ecc_1d: ecc_1d
    :field wdt_1d: wdt_1d
    :field crc_error_1d: crc_error_1d
    :field sc_1d: sc_1d
    :field mem_sel_1_1d: mem_sel_1_1d
    :field mem_sel_2_1d: mem_sel_2_1d
    :field frame_counter_error_1d: frame_counter_error_1d
    :field fm_1d: fm_1d
    :field engineering_data_2a: engineering_data_2a
    :field engineering_data_2b: engineering_data_2b
    :field spin_period: spin_period
    :field magnetorquer_mtq_x_1: magnetorquer_mtq_x_1
    :field magnetorquer_mtq_x_2: magnetorquer_mtq_x_2
    :field magnetorquer_mtq_x_mbc_cont: magnetorquer_mtq_x_mbc_cont
    :field magnetorquer_mtq_x_plus: magnetorquer_mtq_x_plus
    :field magnetorquer_mtq_y_spin_null: magnetorquer_mtq_y_spin_null
    :field magnetorquer_mtq_y_spin: magnetorquer_mtq_y_spin
    :field magnetorquer_mtq_y_1: magnetorquer_mtq_y_1
    :field magnetorquer_mtq_y_2: magnetorquer_mtq_y_2
    :field sun_angle_changed: sun_angle_changed
    :field sun_angle: sun_angle
    :field magnet_sensor_z_axis: magnet_sensor_z_axis
    :field magnet_sensor_x_axis: magnet_sensor_x_axis
    :field current_from_solar_cells: current_from_solar_cells
    :field battery_charge_and_discharge_current: battery_charge_and_discharge_current
    :field battery_voltage: battery_voltage
    :field battery_intermediate_terminal_voltage: battery_intermediate_terminal_voltage
    :field bus_voltage: bus_voltage
    :field jta_tx_power: jta_tx_power
    :field structure_temperature_1: structure_temperature_1
    :field structure_temperature_2: structure_temperature_2
    :field structure_temperature_3: structure_temperature_3
    :field structure_temperature_4: structure_temperature_4
    :field battery_temperature: battery_temperature
    :field beacon: beacon
    :field necessary_for_lengthcheck: necessary_for_lengthcheck
    
    .. seealso::
       Source - https://www.jarl.org/Japanese/3_Fuji/teleme/tlm.htm
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.header_hihi = (self._io.read_bytes(4)).decode(u"ASCII")
        if not self.header_hihi == u"HIHI":
            raise kaitaistruct.ValidationNotEqualError(u"HIHI", self.header_hihi, self._io, u"/seq/0")
        self.byte_1a_ascii = (self._io.read_bytes(2)).decode(u"ASCII")
        self.byte_1b_ascii = (self._io.read_bytes(2)).decode(u"ASCII")
        self.byte_1c_ascii = (self._io.read_bytes(2)).decode(u"ASCII")
        self.byte_1d_ascii = (self._io.read_bytes(2)).decode(u"ASCII")
        self.byte_2a_ascii = (self._io.read_bytes(2)).decode(u"ASCII")
        self.byte_2b_ascii = (self._io.read_bytes(2)).decode(u"ASCII")
        self.byte_2c_ascii = (self._io.read_bytes(2)).decode(u"ASCII")
        self.byte_2d_ascii = (self._io.read_bytes(2)).decode(u"ASCII")
        self.byte_3a_ascii = (self._io.read_bytes(2)).decode(u"ASCII")
        self.byte_3b_ascii = (self._io.read_bytes(2)).decode(u"ASCII")
        self.byte_3c_ascii = (self._io.read_bytes(2)).decode(u"ASCII")
        self.byte_3d_ascii = (self._io.read_bytes(2)).decode(u"ASCII")
        self.byte_4a_ascii = (self._io.read_bytes(2)).decode(u"ASCII")
        self.byte_4b_ascii = (self._io.read_bytes(2)).decode(u"ASCII")
        self.byte_4c_ascii = (self._io.read_bytes(2)).decode(u"ASCII")
        self.byte_4d_ascii = (self._io.read_bytes(2)).decode(u"ASCII")
        self.byte_5a_ascii = (self._io.read_bytes(2)).decode(u"ASCII")
        self.byte_5b_ascii = (self._io.read_bytes(2)).decode(u"ASCII")
        self.byte_5c_ascii = (self._io.read_bytes(2)).decode(u"ASCII")
        self.byte_5d_ascii = (self._io.read_bytes(2)).decode(u"ASCII")
        self.byte_6a_ascii = (self._io.read_bytes(2)).decode(u"ASCII")
        self.byte_6b_ascii = (self._io.read_bytes(2)).decode(u"ASCII")
        self.byte_6c_ascii = (self._io.read_bytes(2)).decode(u"ASCII")
        self.lengthcheck = (self._io.read_bytes_full()).decode(u"utf-8")

    @property
    def digitalker_mode(self):
        if hasattr(self, '_m_digitalker_mode'):
            return self._m_digitalker_mode

        if  (((self.byte_1c_ascii)[0:1] != u"*") and ((self.byte_1c_ascii)[1:2] != u"*")) :
            self._m_digitalker_mode = ((int(self.byte_1c_ascii, 16) & 16) >> 4)

        return getattr(self, '_m_digitalker_mode', None)

    @property
    def battery_temperature(self):
        if hasattr(self, '_m_battery_temperature'):
            return self._m_battery_temperature

        if  (((self.byte_6c_ascii)[0:1] != u"*") and ((self.byte_6c_ascii)[1:2] != u"*")) :
            self._m_battery_temperature = ((int(self.byte_6c_ascii, 16) * -0.388375) + 81.883)

        return getattr(self, '_m_battery_temperature', None)

    @property
    def necessary_for_lengthcheck(self):
        if hasattr(self, '_m_necessary_for_lengthcheck'):
            return self._m_necessary_for_lengthcheck

        if len(self.lengthcheck) != 0:
            self._m_necessary_for_lengthcheck = int(self.lengthcheck) // 0

        return getattr(self, '_m_necessary_for_lengthcheck', None)

    @property
    def jta_tx_power(self):
        if hasattr(self, '_m_jta_tx_power'):
            return self._m_jta_tx_power

        if  (((self.byte_5b_ascii)[0:1] != u"*") and ((self.byte_5b_ascii)[1:2] != u"*")) :
            self._m_jta_tx_power = ((int(self.byte_5b_ascii, 16) * 6.4997) - 98.0863)

        return getattr(self, '_m_jta_tx_power', None)

    @property
    def mem_sel_1_1d(self):
        if hasattr(self, '_m_mem_sel_1_1d'):
            return self._m_mem_sel_1_1d

        if  (((self.byte_1d_ascii)[0:1] != u"*") and ((self.byte_1d_ascii)[1:2] != u"*")) :
            self._m_mem_sel_1_1d = ((int(self.byte_1d_ascii, 16) & 16) >> 4)

        return getattr(self, '_m_mem_sel_1_1d', None)

    @property
    def data_collect_mode(self):
        if hasattr(self, '_m_data_collect_mode'):
            return self._m_data_collect_mode

        if  (((self.byte_1c_ascii)[0:1] != u"*") and ((self.byte_1c_ascii)[1:2] != u"*")) :
            self._m_data_collect_mode = (int(self.byte_1c_ascii, 16) & 1)

        return getattr(self, '_m_data_collect_mode', None)

    @property
    def engineering_data_2a(self):
        if hasattr(self, '_m_engineering_data_2a'):
            return self._m_engineering_data_2a

        if  (((self.byte_2a_ascii)[0:1] != u"*") and ((self.byte_2a_ascii)[1:2] != u"*")) :
            self._m_engineering_data_2a = int(self.byte_2a_ascii, 16)

        return getattr(self, '_m_engineering_data_2a', None)

    @property
    def magnet_sensor_x_axis(self):
        if hasattr(self, '_m_magnet_sensor_x_axis'):
            return self._m_magnet_sensor_x_axis

        if  (((self.byte_3d_ascii)[0:1] != u"*") and ((self.byte_3d_ascii)[1:2] != u"*")) :
            self._m_magnet_sensor_x_axis = (((int(self.byte_3d_ascii, 16) + 102) * 490.196) - 50000)

        return getattr(self, '_m_magnet_sensor_x_axis', None)

    @property
    def packet(self):
        if hasattr(self, '_m_packet'):
            return self._m_packet

        if  (((self.byte_1a_ascii)[0:1] != u"*") and ((self.byte_1a_ascii)[1:2] != u"*")) :
            self._m_packet = ((int(self.byte_1a_ascii, 16) & 24) >> 3)

        return getattr(self, '_m_packet', None)

    @property
    def sun_bin(self):
        if hasattr(self, '_m_sun_bin'):
            return self._m_sun_bin

        if  (((self.byte_3b_ascii)[0:1] != u"*") and ((self.byte_3b_ascii)[1:2] != u"*") and (self.dcm == 1)) :
            self._m_sun_bin = (int(self.byte_3b_ascii, 16) & 127)

        return getattr(self, '_m_sun_bin', None)

    @property
    def battery_voltage(self):
        if hasattr(self, '_m_battery_voltage'):
            return self._m_battery_voltage

        if  (((self.byte_4c_ascii)[0:1] != u"*") and ((self.byte_4c_ascii)[1:2] != u"*")) :
            self._m_battery_voltage = (int(self.byte_4c_ascii, 16) * 0.10761)

        return getattr(self, '_m_battery_voltage', None)

    @property
    def battery_mode(self):
        if hasattr(self, '_m_battery_mode'):
            return self._m_battery_mode

        if  (((self.byte_1b_ascii)[0:1] != u"*") and ((self.byte_1b_ascii)[1:2] != u"*")) :
            self._m_battery_mode = ((int(self.byte_1b_ascii, 16) & 64) >> 6)

        return getattr(self, '_m_battery_mode', None)

    @property
    def crc_error_1d(self):
        if hasattr(self, '_m_crc_error_1d'):
            return self._m_crc_error_1d

        if  (((self.byte_1d_ascii)[0:1] != u"*") and ((self.byte_1d_ascii)[1:2] != u"*")) :
            self._m_crc_error_1d = ((int(self.byte_1d_ascii, 16) & 4) >> 2)

        return getattr(self, '_m_crc_error_1d', None)

    @property
    def spin_period(self):
        if hasattr(self, '_m_spin_period'):
            return self._m_spin_period

        if  (((self.byte_2c_ascii)[0:1] != u"*") and ((self.byte_2c_ascii)[1:2] != u"*") and ((self.byte_2d_ascii)[0:1] != u"*") and ((self.byte_2d_ascii)[1:2] != u"*")) :
            self._m_spin_period = ((((((((((((((((int(self.byte_2c_ascii, 16) & 4) >> 2) * 8192) + (((int(self.byte_2c_ascii, 16) & 8) >> 3) * 4096)) + (((int(self.byte_2c_ascii, 16) & 16) >> 4) * 2048)) + (((int(self.byte_2c_ascii, 16) & 32) >> 5) * 1024)) + (((int(self.byte_2c_ascii, 16) & 64) >> 6) * 512)) + (((int(self.byte_2c_ascii, 16) & 128) >> 7) * 256)) + ((int(self.byte_2d_ascii, 16) & 1) * 128)) + (((int(self.byte_2d_ascii, 16) & 2) >> 1) * 64)) + (((int(self.byte_2d_ascii, 16) & 4) >> 2) * 32)) + (((int(self.byte_2d_ascii, 16) & 8) >> 3) * 16)) + (((int(self.byte_2d_ascii, 16) & 16) >> 4) * 8)) + (((int(self.byte_2d_ascii, 16) & 32) >> 5) * 4)) + (((int(self.byte_2d_ascii, 16) & 64) >> 6) * 2)) + ((int(self.byte_2d_ascii, 16) & 128) >> 7))

        return getattr(self, '_m_spin_period', None)

    @property
    def beacon(self):
        if hasattr(self, '_m_beacon'):
            return self._m_beacon

        self._m_beacon = self.header_hihi + u" " + self.byte_1a_ascii + u" " + self.byte_1b_ascii + u" " + self.byte_1c_ascii + u" " + self.byte_1d_ascii + u" " + self.byte_2a_ascii + u" " + self.byte_2b_ascii + u" " + self.byte_2c_ascii + u" " + self.byte_2d_ascii + u" " + self.byte_3a_ascii + u" " + self.byte_3b_ascii + u" " + self.byte_3c_ascii + u" " + self.byte_3d_ascii + u" " + self.byte_4a_ascii + u" " + self.byte_4b_ascii + u" " + self.byte_4c_ascii + u" " + self.byte_4d_ascii + u" " + self.byte_5a_ascii + u" " + self.byte_5b_ascii + u" " + self.byte_5c_ascii + u" " + self.byte_5d_ascii + u" " + self.byte_6a_ascii + u" " + self.byte_6b_ascii + u" " + self.byte_6c_ascii
        return getattr(self, '_m_beacon', None)

    @property
    def sram(self):
        if hasattr(self, '_m_sram'):
            return self._m_sram

        if  (((self.byte_1a_ascii)[0:1] != u"*") and ((self.byte_1a_ascii)[1:2] != u"*")) :
            self._m_sram = ((int(self.byte_1a_ascii, 16) & 4) >> 2)

        return getattr(self, '_m_sram', None)

    @property
    def frame_counter_error_1d(self):
        if hasattr(self, '_m_frame_counter_error_1d'):
            return self._m_frame_counter_error_1d

        if  (((self.byte_1d_ascii)[0:1] != u"*") and ((self.byte_1d_ascii)[1:2] != u"*")) :
            self._m_frame_counter_error_1d = ((int(self.byte_1d_ascii, 16) & 64) >> 6)

        return getattr(self, '_m_frame_counter_error_1d', None)

    @property
    def main_relay(self):
        if hasattr(self, '_m_main_relay'):
            return self._m_main_relay

        if  (((self.byte_1a_ascii)[0:1] != u"*") and ((self.byte_1a_ascii)[1:2] != u"*")) :
            self._m_main_relay = (int(self.byte_1a_ascii, 16) & 1)

        return getattr(self, '_m_main_relay', None)

    @property
    def magnetorquer_mtq_y_2(self):
        if hasattr(self, '_m_magnetorquer_mtq_y_2'):
            return self._m_magnetorquer_mtq_y_2

        if  (((self.byte_3a_ascii)[0:1] != u"*") and ((self.byte_3a_ascii)[1:2] != u"*")) :
            self._m_magnetorquer_mtq_y_2 = ((int(self.byte_3a_ascii, 16) & 128) >> 7)

        return getattr(self, '_m_magnetorquer_mtq_y_2', None)

    @property
    def ecc_1d(self):
        if hasattr(self, '_m_ecc_1d'):
            return self._m_ecc_1d

        if  (((self.byte_1d_ascii)[0:1] != u"*") and ((self.byte_1d_ascii)[1:2] != u"*")) :
            self._m_ecc_1d = (int(self.byte_1d_ascii, 16) & 1)

        return getattr(self, '_m_ecc_1d', None)

    @property
    def magnetorquer_mtq_x_2(self):
        if hasattr(self, '_m_magnetorquer_mtq_x_2'):
            return self._m_magnetorquer_mtq_x_2

        if  (((self.byte_3a_ascii)[0:1] != u"*") and ((self.byte_3a_ascii)[1:2] != u"*")) :
            self._m_magnetorquer_mtq_x_2 = ((int(self.byte_3a_ascii, 16) & 2) >> 1)

        return getattr(self, '_m_magnetorquer_mtq_x_2', None)

    @property
    def magnetorquer_mtq_x_mbc_cont(self):
        if hasattr(self, '_m_magnetorquer_mtq_x_mbc_cont'):
            return self._m_magnetorquer_mtq_x_mbc_cont

        if  (((self.byte_3a_ascii)[0:1] != u"*") and ((self.byte_3a_ascii)[1:2] != u"*")) :
            self._m_magnetorquer_mtq_x_mbc_cont = ((int(self.byte_3a_ascii, 16) & 4) >> 2)

        return getattr(self, '_m_magnetorquer_mtq_x_mbc_cont', None)

    @property
    def current_from_solar_cells(self):
        if hasattr(self, '_m_current_from_solar_cells'):
            return self._m_current_from_solar_cells

        if  (((self.byte_4a_ascii)[0:1] != u"*") and ((self.byte_4a_ascii)[1:2] != u"*")) :
            self._m_current_from_solar_cells = (int(self.byte_4a_ascii, 16) * 0.009804)

        return getattr(self, '_m_current_from_solar_cells', None)

    @property
    def pcu_level(self):
        if hasattr(self, '_m_pcu_level'):
            return self._m_pcu_level

        if  (((self.byte_1b_ascii)[0:1] != u"*") and ((self.byte_1b_ascii)[1:2] != u"*")) :
            self._m_pcu_level = ((int(self.byte_1b_ascii, 16) & 48) >> 4)

        return getattr(self, '_m_pcu_level', None)

    @property
    def packet_hk_date_mode(self):
        if hasattr(self, '_m_packet_hk_date_mode'):
            return self._m_packet_hk_date_mode

        if  (((self.byte_1c_ascii)[0:1] != u"*") and ((self.byte_1c_ascii)[1:2] != u"*")) :
            self._m_packet_hk_date_mode = ((int(self.byte_1c_ascii, 16) & 4) >> 2)

        return getattr(self, '_m_packet_hk_date_mode', None)

    @property
    def fm_1d(self):
        if hasattr(self, '_m_fm_1d'):
            return self._m_fm_1d

        if  (((self.byte_1d_ascii)[0:1] != u"*") and ((self.byte_1d_ascii)[1:2] != u"*")) :
            self._m_fm_1d = ((int(self.byte_1d_ascii, 16) & 128) >> 7)

        return getattr(self, '_m_fm_1d', None)

    @property
    def repro_mode(self):
        if hasattr(self, '_m_repro_mode'):
            return self._m_repro_mode

        if  (((self.byte_1c_ascii)[0:1] != u"*") and ((self.byte_1c_ascii)[1:2] != u"*")) :
            self._m_repro_mode = ((int(self.byte_1c_ascii, 16) & 2) >> 1)

        return getattr(self, '_m_repro_mode', None)

    @property
    def engineering_data_2b(self):
        if hasattr(self, '_m_engineering_data_2b'):
            return self._m_engineering_data_2b

        if  (((self.byte_2b_ascii)[0:1] != u"*") and ((self.byte_2b_ascii)[1:2] != u"*")) :
            self._m_engineering_data_2b = int(self.byte_2b_ascii, 16)

        return getattr(self, '_m_engineering_data_2b', None)

    @property
    def geomagnetism_sensor(self):
        if hasattr(self, '_m_geomagnetism_sensor'):
            return self._m_geomagnetism_sensor

        if  (((self.byte_1a_ascii)[0:1] != u"*") and ((self.byte_1a_ascii)[1:2] != u"*")) :
            self._m_geomagnetism_sensor = ((int(self.byte_1a_ascii, 16) & 128) >> 7)

        return getattr(self, '_m_geomagnetism_sensor', None)

    @property
    def packet_date_collect_mode(self):
        if hasattr(self, '_m_packet_date_collect_mode'):
            return self._m_packet_date_collect_mode

        if  (((self.byte_1c_ascii)[0:1] != u"*") and ((self.byte_1c_ascii)[1:2] != u"*")) :
            self._m_packet_date_collect_mode = ((int(self.byte_1c_ascii, 16) & 8) >> 3)

        return getattr(self, '_m_packet_date_collect_mode', None)

    @property
    def jtd(self):
        if hasattr(self, '_m_jtd'):
            return self._m_jtd

        if  (((self.byte_1a_ascii)[0:1] != u"*") and ((self.byte_1a_ascii)[1:2] != u"*")) :
            self._m_jtd = ((int(self.byte_1a_ascii, 16) & 64) >> 6)

        return getattr(self, '_m_jtd', None)

    @property
    def mem_sel_2_1d(self):
        if hasattr(self, '_m_mem_sel_2_1d'):
            return self._m_mem_sel_2_1d

        if  (((self.byte_1d_ascii)[0:1] != u"*") and ((self.byte_1d_ascii)[1:2] != u"*")) :
            self._m_mem_sel_2_1d = ((int(self.byte_1d_ascii, 16) & 32) >> 5)

        return getattr(self, '_m_mem_sel_2_1d', None)

    @property
    def battery_intermediate_terminal_voltage(self):
        if hasattr(self, '_m_battery_intermediate_terminal_voltage'):
            return self._m_battery_intermediate_terminal_voltage

        if  (((self.byte_4d_ascii)[0:1] != u"*") and ((self.byte_4d_ascii)[1:2] != u"*")) :
            self._m_battery_intermediate_terminal_voltage = (int(self.byte_4d_ascii, 16) * 0.04817)

        return getattr(self, '_m_battery_intermediate_terminal_voltage', None)

    @property
    def jta(self):
        if hasattr(self, '_m_jta'):
            return self._m_jta

        if  (((self.byte_1a_ascii)[0:1] != u"*") and ((self.byte_1a_ascii)[1:2] != u"*")) :
            self._m_jta = ((int(self.byte_1a_ascii, 16) & 32) >> 5)

        return getattr(self, '_m_jta', None)

    @property
    def magnetorquer_mtq_x_1(self):
        if hasattr(self, '_m_magnetorquer_mtq_x_1'):
            return self._m_magnetorquer_mtq_x_1

        if  (((self.byte_3a_ascii)[0:1] != u"*") and ((self.byte_3a_ascii)[1:2] != u"*")) :
            self._m_magnetorquer_mtq_x_1 = (int(self.byte_3a_ascii, 16) & 1)

        return getattr(self, '_m_magnetorquer_mtq_x_1', None)

    @property
    def structure_temperature_4(self):
        if hasattr(self, '_m_structure_temperature_4'):
            return self._m_structure_temperature_4

        if  (((self.byte_6b_ascii)[0:1] != u"*") and ((self.byte_6b_ascii)[1:2] != u"*")) :
            self._m_structure_temperature_4 = ((int(self.byte_6b_ascii, 16) * -0.388375) + 81.883)

        return getattr(self, '_m_structure_temperature_4', None)

    @property
    def structure_temperature_3(self):
        if hasattr(self, '_m_structure_temperature_3'):
            return self._m_structure_temperature_3

        if  (((self.byte_6a_ascii)[0:1] != u"*") and ((self.byte_6a_ascii)[1:2] != u"*")) :
            self._m_structure_temperature_3 = ((int(self.byte_6a_ascii, 16) * -0.388375) + 81.883)

        return getattr(self, '_m_structure_temperature_3', None)

    @property
    def double_cmd_ena_dis(self):
        if hasattr(self, '_m_double_cmd_ena_dis'):
            return self._m_double_cmd_ena_dis

        if  (((self.byte_1c_ascii)[0:1] != u"*") and ((self.byte_1c_ascii)[1:2] != u"*")) :
            self._m_double_cmd_ena_dis = ((int(self.byte_1c_ascii, 16) & 32) >> 5)

        return getattr(self, '_m_double_cmd_ena_dis', None)

    @property
    def wdt_1d(self):
        if hasattr(self, '_m_wdt_1d'):
            return self._m_wdt_1d

        if  (((self.byte_1d_ascii)[0:1] != u"*") and ((self.byte_1d_ascii)[1:2] != u"*")) :
            self._m_wdt_1d = ((int(self.byte_1d_ascii, 16) & 2) >> 1)

        return getattr(self, '_m_wdt_1d', None)

    @property
    def structure_temperature_1(self):
        if hasattr(self, '_m_structure_temperature_1'):
            return self._m_structure_temperature_1

        if  (((self.byte_5c_ascii)[0:1] != u"*") and ((self.byte_5c_ascii)[1:2] != u"*")) :
            self._m_structure_temperature_1 = ((int(self.byte_5c_ascii, 16) * -0.388375) + 81.883)

        return getattr(self, '_m_structure_temperature_1', None)

    @property
    def sc_1d(self):
        if hasattr(self, '_m_sc_1d'):
            return self._m_sc_1d

        if  (((self.byte_1d_ascii)[0:1] != u"*") and ((self.byte_1d_ascii)[1:2] != u"*")) :
            self._m_sc_1d = ((int(self.byte_1d_ascii, 16) & 8) >> 3)

        return getattr(self, '_m_sc_1d', None)

    @property
    def uvc(self):
        if hasattr(self, '_m_uvc'):
            return self._m_uvc

        if  (((self.byte_1b_ascii)[0:1] != u"*") and ((self.byte_1b_ascii)[1:2] != u"*")) :
            self._m_uvc = ((int(self.byte_1b_ascii, 16) & 2) >> 1)

        return getattr(self, '_m_uvc', None)

    @property
    def magnetorquer_mtq_y_spin_null(self):
        if hasattr(self, '_m_magnetorquer_mtq_y_spin_null'):
            return self._m_magnetorquer_mtq_y_spin_null

        if  (((self.byte_3a_ascii)[0:1] != u"*") and ((self.byte_3a_ascii)[1:2] != u"*")) :
            self._m_magnetorquer_mtq_y_spin_null = ((int(self.byte_3a_ascii, 16) & 16) >> 4)

        return getattr(self, '_m_magnetorquer_mtq_y_spin_null', None)

    @property
    def uvc_level(self):
        if hasattr(self, '_m_uvc_level'):
            return self._m_uvc_level

        if  (((self.byte_1b_ascii)[0:1] != u"*") and ((self.byte_1b_ascii)[1:2] != u"*")) :
            self._m_uvc_level = ((int(self.byte_1b_ascii, 16) & 4) >> 2)

        return getattr(self, '_m_uvc_level', None)

    @property
    def magnetorquer_mtq_y_1(self):
        if hasattr(self, '_m_magnetorquer_mtq_y_1'):
            return self._m_magnetorquer_mtq_y_1

        if  (((self.byte_3a_ascii)[0:1] != u"*") and ((self.byte_3a_ascii)[1:2] != u"*")) :
            self._m_magnetorquer_mtq_y_1 = ((int(self.byte_3a_ascii, 16) & 64) >> 6)

        return getattr(self, '_m_magnetorquer_mtq_y_1', None)

    @property
    def cpu_run_reset(self):
        if hasattr(self, '_m_cpu_run_reset'):
            return self._m_cpu_run_reset

        if  (((self.byte_1c_ascii)[0:1] != u"*") and ((self.byte_1c_ascii)[1:2] != u"*")) :
            self._m_cpu_run_reset = ((int(self.byte_1c_ascii, 16) & 128) >> 7)

        return getattr(self, '_m_cpu_run_reset', None)

    @property
    def uvc_active_passive(self):
        if hasattr(self, '_m_uvc_active_passive'):
            return self._m_uvc_active_passive

        if  (((self.byte_1c_ascii)[0:1] != u"*") and ((self.byte_1c_ascii)[1:2] != u"*")) :
            self._m_uvc_active_passive = ((int(self.byte_1c_ascii, 16) & 64) >> 6)

        return getattr(self, '_m_uvc_active_passive', None)

    @property
    def bus_voltage(self):
        if hasattr(self, '_m_bus_voltage'):
            return self._m_bus_voltage

        if  (((self.byte_5a_ascii)[0:1] != u"*") and ((self.byte_5a_ascii)[1:2] != u"*")) :
            self._m_bus_voltage = (int(self.byte_5a_ascii, 16) * 0.09804)

        return getattr(self, '_m_bus_voltage', None)

    @property
    def pcu_mode(self):
        if hasattr(self, '_m_pcu_mode'):
            return self._m_pcu_mode

        if  (((self.byte_1b_ascii)[0:1] != u"*") and ((self.byte_1b_ascii)[1:2] != u"*")) :
            self._m_pcu_mode = ((int(self.byte_1b_ascii, 16) & 8) >> 3)

        return getattr(self, '_m_pcu_mode', None)

    @property
    def sun_angle_changed(self):
        if hasattr(self, '_m_sun_angle_changed'):
            return self._m_sun_angle_changed

        if  (((self.byte_3b_ascii)[0:1] != u"*") and ((self.byte_3b_ascii)[1:2] != u"*") and (self.dcm == 1)) :
            self._m_sun_angle_changed = ((int(self.byte_3b_ascii, 16) & 128) >> 7)

        return getattr(self, '_m_sun_angle_changed', None)

    @property
    def magnet_sensor_z_axis(self):
        if hasattr(self, '_m_magnet_sensor_z_axis'):
            return self._m_magnet_sensor_z_axis

        if  (((self.byte_3c_ascii)[0:1] != u"*") and ((self.byte_3c_ascii)[1:2] != u"*")) :
            self._m_magnet_sensor_z_axis = (int(self.byte_3c_ascii, 16) * 490.196)

        return getattr(self, '_m_magnet_sensor_z_axis', None)

    @property
    def battery_charge_and_discharge_current(self):
        if hasattr(self, '_m_battery_charge_and_discharge_current'):
            return self._m_battery_charge_and_discharge_current

        if  (((self.byte_4b_ascii)[0:1] != u"*") and ((self.byte_4b_ascii)[1:2] != u"*")) :
            self._m_battery_charge_and_discharge_current = -((2 - (int(self.byte_4b_ascii, 16) * 0.0196)))

        return getattr(self, '_m_battery_charge_and_discharge_current', None)

    @property
    def structure_temperature_2(self):
        if hasattr(self, '_m_structure_temperature_2'):
            return self._m_structure_temperature_2

        if  (((self.byte_5d_ascii)[0:1] != u"*") and ((self.byte_5d_ascii)[1:2] != u"*")) :
            self._m_structure_temperature_2 = ((int(self.byte_5d_ascii, 16) * -0.388375) + 81.883)

        return getattr(self, '_m_structure_temperature_2', None)

    @property
    def sun_angle(self):
        if hasattr(self, '_m_sun_angle'):
            return self._m_sun_angle

        if  (((self.byte_3b_ascii)[0:1] != u"*") and ((self.byte_3b_ascii)[1:2] != u"*") and (self.dcm == 1)) :
            self._m_sun_angle = (27.5 if self.sun_bin == 1 else (28.5 if self.sun_bin == 3 else (29.5 if self.sun_bin == 2 else (30.5 if self.sun_bin == 6 else (31.5 if self.sun_bin == 7 else (32.5 if self.sun_bin == 5 else (33.5 if self.sun_bin == 4 else (34.5 if self.sun_bin == 12 else (35.5 if self.sun_bin == 13 else (36.5 if self.sun_bin == 15 else (37.5 if self.sun_bin == 14 else (38.5 if self.sun_bin == 10 else (39.5 if self.sun_bin == 11 else (40.5 if self.sun_bin == 9 else (41.5 if self.sun_bin == 8 else (42.5 if self.sun_bin == 24 else (43.5 if self.sun_bin == 25 else (44.5 if self.sun_bin == 27 else (45.5 if self.sun_bin == 26 else (46.5 if self.sun_bin == 30 else (47.5 if self.sun_bin == 31 else (48.5 if self.sun_bin == 29 else (49.5 if self.sun_bin == 28 else (50.5 if self.sun_bin == 20 else (51.5 if self.sun_bin == 21 else (52.5 if self.sun_bin == 23 else (53.5 if self.sun_bin == 22 else (54.5 if self.sun_bin == 18 else (55.5 if self.sun_bin == 19 else (56.5 if self.sun_bin == 17 else (57.5 if self.sun_bin == 16 else (58.5 if self.sun_bin == 48 else (59.5 if self.sun_bin == 49 else (60.5 if self.sun_bin == 51 else (61.5 if self.sun_bin == 50 else (62.5 if self.sun_bin == 54 else (63.5 if self.sun_bin == 55 else (64.5 if self.sun_bin == 53 else (65.5 if self.sun_bin == 52 else (66.5 if self.sun_bin == 60 else (67.5 if self.sun_bin == 61 else (68.5 if self.sun_bin == 63 else (69.5 if self.sun_bin == 62 else (70.5 if self.sun_bin == 58 else (71.5 if self.sun_bin == 59 else (72.5 if self.sun_bin == 57 else (73.5 if self.sun_bin == 56 else (74.5 if self.sun_bin == 40 else (75.5 if self.sun_bin == 41 else (76.5 if self.sun_bin == 43 else (77.5 if self.sun_bin == 42 else (78.5 if self.sun_bin == 46 else (79.5 if self.sun_bin == 47 else (80.5 if self.sun_bin == 45 else (81.5 if self.sun_bin == 44 else (82.5 if self.sun_bin == 36 else (83.5 if self.sun_bin == 37 else (84.5 if self.sun_bin == 39 else (85.5 if self.sun_bin == 38 else (86.5 if self.sun_bin == 34 else (87.5 if self.sun_bin == 35 else (88.5 if self.sun_bin == 33 else (89.5 if self.sun_bin == 32 else (90.5 if self.sun_bin == 96 else (91.5 if self.sun_bin == 97 else (92.5 if self.sun_bin == 99 else (93.5 if self.sun_bin == 98 else (94.5 if self.sun_bin == 102 else (95.5 if self.sun_bin == 103 else (96.5 if self.sun_bin == 101 else (97.5 if self.sun_bin == 100 else (98.5 if self.sun_bin == 108 else (99.5 if self.sun_bin == 109 else (100.5 if self.sun_bin == 111 else (101.5 if self.sun_bin == 110 else (102.5 if self.sun_bin == 106 else (103.5 if self.sun_bin == 107 else (104.5 if self.sun_bin == 105 else (105.5 if self.sun_bin == 104 else (106.5 if self.sun_bin == 120 else (107.5 if self.sun_bin == 121 else (108.5 if self.sun_bin == 123 else (109.5 if self.sun_bin == 122 else (110.5 if self.sun_bin == 126 else (111.5 if self.sun_bin == 127 else (112.5 if self.sun_bin == 125 else (113.5 if self.sun_bin == 124 else (114.5 if self.sun_bin == 116 else (115.5 if self.sun_bin == 117 else (116.5 if self.sun_bin == 119 else (117.5 if self.sun_bin == 118 else (118.5 if self.sun_bin == 114 else (119.5 if self.sun_bin == 115 else (120.5 if self.sun_bin == 113 else (121.5 if self.sun_bin == 112 else (122.5 if self.sun_bin == 80 else (123.5 if self.sun_bin == 81 else (124.5 if self.sun_bin == 83 else (125.5 if self.sun_bin == 82 else (126.5 if self.sun_bin == 86 else (127.5 if self.sun_bin == 87 else (128.5 if self.sun_bin == 85 else (129.5 if self.sun_bin == 84 else (130.5 if self.sun_bin == 92 else (131.5 if self.sun_bin == 93 else (132.5 if self.sun_bin == 95 else (133.5 if self.sun_bin == 94 else (134.5 if self.sun_bin == 90 else (135.5 if self.sun_bin == 91 else (136.5 if self.sun_bin == 89 else (137.5 if self.sun_bin == 88 else (138.5 if self.sun_bin == 72 else (139.5 if self.sun_bin == 73 else (140.5 if self.sun_bin == 75 else (141.5 if self.sun_bin == 74 else (142.5 if self.sun_bin == 78 else (143.5 if self.sun_bin == 79 else (144.5 if self.sun_bin == 77 else (145.5 if self.sun_bin == 76 else (146.5 if self.sun_bin == 68 else (147.5 if self.sun_bin == 69 else (148.5 if self.sun_bin == 71 else (149.5 if self.sun_bin == 70 else (150.5 if self.sun_bin == 66 else (151.5 if self.sun_bin == 67 else (152.5 if self.sun_bin == 65 else (153.5 if self.sun_bin == 64 else 0)))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))

        return getattr(self, '_m_sun_angle', None)

    @property
    def sun_sensor(self):
        if hasattr(self, '_m_sun_sensor'):
            return self._m_sun_sensor

        if  (((self.byte_1b_ascii)[0:1] != u"*") and ((self.byte_1b_ascii)[1:2] != u"*")) :
            self._m_sun_sensor = (int(self.byte_1b_ascii, 16) & 1)

        return getattr(self, '_m_sun_sensor', None)

    @property
    def dcm(self):
        if hasattr(self, '_m_dcm'):
            return self._m_dcm

        if  (((self.byte_1a_ascii)[0:1] != u"*") and ((self.byte_1a_ascii)[1:2] != u"*")) :
            self._m_dcm = ((int(self.byte_1a_ascii, 16) & 2) >> 1)

        return getattr(self, '_m_dcm', None)

    @property
    def magnetorquer_mtq_y_spin(self):
        if hasattr(self, '_m_magnetorquer_mtq_y_spin'):
            return self._m_magnetorquer_mtq_y_spin

        if  (((self.byte_3a_ascii)[0:1] != u"*") and ((self.byte_3a_ascii)[1:2] != u"*")) :
            self._m_magnetorquer_mtq_y_spin = ((int(self.byte_3a_ascii, 16) & 32) >> 5)

        return getattr(self, '_m_magnetorquer_mtq_y_spin', None)

    @property
    def battery_logic(self):
        if hasattr(self, '_m_battery_logic'):
            return self._m_battery_logic

        if  (((self.byte_1b_ascii)[0:1] != u"*") and ((self.byte_1b_ascii)[1:2] != u"*")) :
            self._m_battery_logic = ((int(self.byte_1b_ascii, 16) & 128) >> 7)

        return getattr(self, '_m_battery_logic', None)

    @property
    def magnetorquer_mtq_x_plus(self):
        if hasattr(self, '_m_magnetorquer_mtq_x_plus'):
            return self._m_magnetorquer_mtq_x_plus

        if  (((self.byte_3a_ascii)[0:1] != u"*") and ((self.byte_3a_ascii)[1:2] != u"*")) :
            self._m_magnetorquer_mtq_x_plus = ((int(self.byte_3a_ascii, 16) & 8) >> 3)

        return getattr(self, '_m_magnetorquer_mtq_x_plus', None)





# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if getattr(kaitaistruct, 'API_VERSION', (0, 9)) < (0, 9):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Teikyosat4(KaitaiStruct):
    """:field s: s
    :field rssi_vhf: rssi_vhf
    :field rssi_shf: rssi_shf
    :field debug_connector: debug_connector
    :field gnd: gnd
    :field plus_x: plus_x
    :field plus_y: plus_y
    :field minus_x: minus_x
    :field minus_y: minus_y
    :field necessary_for_lengthcheck: necessary_for_lengthcheck
    :field beacon: beacon
    
    .. seealso::
       Source - https://drive.usercontent.google.com/download?id=1pdAPBraX38qtzIzjj6hehnZAioILPKhL&export=download
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.s = (self._io.read_bytes(2)).decode(u"ASCII")
        if not  ((self.s == u"s1") or (self.s == u"s2") or (self.s == u"S1") or (self.s == u"S2")) :
            raise kaitaistruct.ValidationNotAnyOfError(self.s, self._io, u"/seq/0")
        self.value1 = self._io.read_u1()
        self.value2 = self._io.read_u1()
        self.value3 = self._io.read_u1()
        self.value4 = self._io.read_u1()
        self.lengthcheck = (self._io.read_bytes_full()).decode(u"utf-8")

    @property
    def debug_connector(self):
        if hasattr(self, '_m_debug_connector'):
            return self._m_debug_connector

        if  ((self.s == u"s1") or (self.s == u"S1")) :
            self._m_debug_connector = self.value3

        return getattr(self, '_m_debug_connector', None)

    @property
    def value3_hex_right_digit(self):
        if hasattr(self, '_m_value3_hex_right_digit'):
            return self._m_value3_hex_right_digit

        self._m_value3_hex_right_digit = (u"a" if str(self.value3_hex_right) == u"10" else (u"b" if str(self.value3_hex_right) == u"11" else (u"c" if str(self.value3_hex_right) == u"12" else (u"d" if str(self.value3_hex_right) == u"13" else (u"e" if str(self.value3_hex_right) == u"14" else (u"f" if str(self.value3_hex_right) == u"15" else str(self.value3_hex_right)))))))
        return getattr(self, '_m_value3_hex_right_digit', None)

    @property
    def necessary_for_lengthcheck(self):
        if hasattr(self, '_m_necessary_for_lengthcheck'):
            return self._m_necessary_for_lengthcheck

        if len(self.lengthcheck) != 0:
            self._m_necessary_for_lengthcheck = int(self.lengthcheck) // 0

        return getattr(self, '_m_necessary_for_lengthcheck', None)

    @property
    def value1_hex_left(self):
        if hasattr(self, '_m_value1_hex_left'):
            return self._m_value1_hex_left

        self._m_value1_hex_left = self.value1 // 16
        return getattr(self, '_m_value1_hex_left', None)

    @property
    def minus_x(self):
        if hasattr(self, '_m_minus_x'):
            return self._m_minus_x

        if  ((self.s == u"s2") or (self.s == u"S2")) :
            self._m_minus_x = ((self.value3 * 5.00) / 255)

        return getattr(self, '_m_minus_x', None)

    @property
    def plus_y(self):
        if hasattr(self, '_m_plus_y'):
            return self._m_plus_y

        if  ((self.s == u"s2") or (self.s == u"S2")) :
            self._m_plus_y = ((self.value2 * 5.00) / 255)

        return getattr(self, '_m_plus_y', None)

    @property
    def value2_hex_right(self):
        if hasattr(self, '_m_value2_hex_right'):
            return self._m_value2_hex_right

        self._m_value2_hex_right = (self.value2 % 16)
        return getattr(self, '_m_value2_hex_right', None)

    @property
    def value3_hex(self):
        if hasattr(self, '_m_value3_hex'):
            return self._m_value3_hex

        self._m_value3_hex = (u".." if self.value3_hex_left_digit + self.value3_hex_right_digit == u"ff" else self.value3_hex_left_digit + self.value3_hex_right_digit)
        return getattr(self, '_m_value3_hex', None)

    @property
    def value3_hex_right(self):
        if hasattr(self, '_m_value3_hex_right'):
            return self._m_value3_hex_right

        self._m_value3_hex_right = (self.value3 % 16)
        return getattr(self, '_m_value3_hex_right', None)

    @property
    def value1_hex_right_digit(self):
        if hasattr(self, '_m_value1_hex_right_digit'):
            return self._m_value1_hex_right_digit

        self._m_value1_hex_right_digit = (u"a" if str(self.value1_hex_right) == u"10" else (u"b" if str(self.value1_hex_right) == u"11" else (u"c" if str(self.value1_hex_right) == u"12" else (u"d" if str(self.value1_hex_right) == u"13" else (u"e" if str(self.value1_hex_right) == u"14" else (u"f" if str(self.value1_hex_right) == u"15" else str(self.value1_hex_right)))))))
        return getattr(self, '_m_value1_hex_right_digit', None)

    @property
    def beacon(self):
        if hasattr(self, '_m_beacon'):
            return self._m_beacon

        self._m_beacon = self.s + u" " + self.value1_hex + u" " + self.value2_hex + u" " + self.value3_hex + u" " + self.value4_hex
        return getattr(self, '_m_beacon', None)

    @property
    def value3_hex_left(self):
        if hasattr(self, '_m_value3_hex_left'):
            return self._m_value3_hex_left

        self._m_value3_hex_left = self.value3 // 16
        return getattr(self, '_m_value3_hex_left', None)

    @property
    def rssi_shf(self):
        if hasattr(self, '_m_rssi_shf'):
            return self._m_rssi_shf

        if  ((self.s == u"s1") or (self.s == u"S1")) :
            self._m_rssi_shf = ((self.value2 * 5.00) / 255)

        return getattr(self, '_m_rssi_shf', None)

    @property
    def value4_hex_left_digit(self):
        if hasattr(self, '_m_value4_hex_left_digit'):
            return self._m_value4_hex_left_digit

        self._m_value4_hex_left_digit = (u"a" if str(self.value4_hex_left) == u"10" else (u"b" if str(self.value4_hex_left) == u"11" else (u"c" if str(self.value4_hex_left) == u"12" else (u"d" if str(self.value4_hex_left) == u"13" else (u"e" if str(self.value4_hex_left) == u"14" else (u"f" if str(self.value4_hex_left) == u"15" else str(self.value4_hex_left)))))))
        return getattr(self, '_m_value4_hex_left_digit', None)

    @property
    def value1_hex_right(self):
        if hasattr(self, '_m_value1_hex_right'):
            return self._m_value1_hex_right

        self._m_value1_hex_right = (self.value1 % 16)
        return getattr(self, '_m_value1_hex_right', None)

    @property
    def value4_hex_right(self):
        if hasattr(self, '_m_value4_hex_right'):
            return self._m_value4_hex_right

        self._m_value4_hex_right = (self.value4 % 16)
        return getattr(self, '_m_value4_hex_right', None)

    @property
    def value4_hex_right_digit(self):
        if hasattr(self, '_m_value4_hex_right_digit'):
            return self._m_value4_hex_right_digit

        self._m_value4_hex_right_digit = (u"a" if str(self.value4_hex_right) == u"10" else (u"b" if str(self.value4_hex_right) == u"11" else (u"c" if str(self.value4_hex_right) == u"12" else (u"d" if str(self.value4_hex_right) == u"13" else (u"e" if str(self.value4_hex_right) == u"14" else (u"f" if str(self.value4_hex_right) == u"15" else str(self.value4_hex_right)))))))
        return getattr(self, '_m_value4_hex_right_digit', None)

    @property
    def value1_hex(self):
        if hasattr(self, '_m_value1_hex'):
            return self._m_value1_hex

        self._m_value1_hex = (u".." if self.value1_hex_left_digit + self.value1_hex_right_digit == u"ff" else self.value1_hex_left_digit + self.value1_hex_right_digit)
        return getattr(self, '_m_value1_hex', None)

    @property
    def gnd(self):
        if hasattr(self, '_m_gnd'):
            return self._m_gnd

        if  ((self.s == u"s1") or (self.s == u"S1")) :
            self._m_gnd = self.value4

        return getattr(self, '_m_gnd', None)

    @property
    def value2_hex_right_digit(self):
        if hasattr(self, '_m_value2_hex_right_digit'):
            return self._m_value2_hex_right_digit

        self._m_value2_hex_right_digit = (u"a" if str(self.value2_hex_right) == u"10" else (u"b" if str(self.value2_hex_right) == u"11" else (u"c" if str(self.value2_hex_right) == u"12" else (u"d" if str(self.value2_hex_right) == u"13" else (u"e" if str(self.value2_hex_right) == u"14" else (u"f" if str(self.value2_hex_right) == u"15" else str(self.value2_hex_right)))))))
        return getattr(self, '_m_value2_hex_right_digit', None)

    @property
    def value4_hex_left(self):
        if hasattr(self, '_m_value4_hex_left'):
            return self._m_value4_hex_left

        self._m_value4_hex_left = self.value4 // 16
        return getattr(self, '_m_value4_hex_left', None)

    @property
    def value2_hex_left_digit(self):
        if hasattr(self, '_m_value2_hex_left_digit'):
            return self._m_value2_hex_left_digit

        self._m_value2_hex_left_digit = (u"a" if str(self.value2_hex_left) == u"10" else (u"b" if str(self.value2_hex_left) == u"11" else (u"c" if str(self.value2_hex_left) == u"12" else (u"d" if str(self.value2_hex_left) == u"13" else (u"e" if str(self.value2_hex_left) == u"14" else (u"f" if str(self.value2_hex_left) == u"15" else str(self.value2_hex_left)))))))
        return getattr(self, '_m_value2_hex_left_digit', None)

    @property
    def plus_x(self):
        if hasattr(self, '_m_plus_x'):
            return self._m_plus_x

        if  ((self.s == u"s2") or (self.s == u"S2")) :
            self._m_plus_x = ((self.value1 * 5.00) / 255)

        return getattr(self, '_m_plus_x', None)

    @property
    def value2_hex(self):
        if hasattr(self, '_m_value2_hex'):
            return self._m_value2_hex

        self._m_value2_hex = (u".." if self.value2_hex_left_digit + self.value2_hex_right_digit == u"ff" else self.value2_hex_left_digit + self.value2_hex_right_digit)
        return getattr(self, '_m_value2_hex', None)

    @property
    def value4_hex(self):
        if hasattr(self, '_m_value4_hex'):
            return self._m_value4_hex

        self._m_value4_hex = (u".." if self.value4_hex_left_digit + self.value4_hex_right_digit == u"ff" else self.value4_hex_left_digit + self.value4_hex_right_digit)
        return getattr(self, '_m_value4_hex', None)

    @property
    def value3_hex_left_digit(self):
        if hasattr(self, '_m_value3_hex_left_digit'):
            return self._m_value3_hex_left_digit

        self._m_value3_hex_left_digit = (u"a" if str(self.value3_hex_left) == u"10" else (u"b" if str(self.value3_hex_left) == u"11" else (u"c" if str(self.value3_hex_left) == u"12" else (u"d" if str(self.value3_hex_left) == u"13" else (u"e" if str(self.value3_hex_left) == u"14" else (u"f" if str(self.value3_hex_left) == u"15" else str(self.value3_hex_left)))))))
        return getattr(self, '_m_value3_hex_left_digit', None)

    @property
    def minus_y(self):
        if hasattr(self, '_m_minus_y'):
            return self._m_minus_y

        if  ((self.s == u"s2") or (self.s == u"S2")) :
            self._m_minus_y = ((self.value4 * 5.00) / 255)

        return getattr(self, '_m_minus_y', None)

    @property
    def value2_hex_left(self):
        if hasattr(self, '_m_value2_hex_left'):
            return self._m_value2_hex_left

        self._m_value2_hex_left = self.value2 // 16
        return getattr(self, '_m_value2_hex_left', None)

    @property
    def rssi_vhf(self):
        if hasattr(self, '_m_rssi_vhf'):
            return self._m_rssi_vhf

        if  ((self.s == u"s1") or (self.s == u"S1")) :
            self._m_rssi_vhf = ((self.value1 * 5.00) / 255)

        return getattr(self, '_m_rssi_vhf', None)

    @property
    def value1_hex_left_digit(self):
        if hasattr(self, '_m_value1_hex_left_digit'):
            return self._m_value1_hex_left_digit

        self._m_value1_hex_left_digit = (u"a" if str(self.value1_hex_left) == u"10" else (u"b" if str(self.value1_hex_left) == u"11" else (u"c" if str(self.value1_hex_left) == u"12" else (u"d" if str(self.value1_hex_left) == u"13" else (u"e" if str(self.value1_hex_left) == u"14" else (u"f" if str(self.value1_hex_left) == u"15" else str(self.value1_hex_left)))))))
        return getattr(self, '_m_value1_hex_left_digit', None)


